TRADE_METADATA = {
    "trade_exports_sector_value": {
        "start_col": 3,
        "currency": "USD",
        "unit": "Millions",
        "colnames": {
            "TOTAL EXPORTACIONES ": "Total exportaciones",
            "ACTIVIDADES PRIMARIAS": "Actividades primarias: total",
            "Agricultura, Ganadería, Caza y actividades de servicios conexas": "Act. primarias: agricultura, ganadería y caza",
            "Cultivos en general; horticultura; servicios agrícolas aplicados a estos cultivos": "Act. primarias: agricultura",
            "Cría de animales; caza ordinaria y servicios conexos; producción de pieles finas mediante actividades de caza ordinaria y con trampas; servicios ganaderos": "Act. primarias: ganadería y caza",
            "Silvicultura, extracción de madera y actividades de servicios conexas": "Act. primarias: silvicultura",
            "INDUSTRIAS MANUFACTURERAS": "Industrias manufactureras: total",
            "Elaboración de productos alimenticios y bebidas": "Ind. manufacturera: alimentos y bebidas",
            "Producción, procesamiento y conservación de carne y productos cárnicos": "Ind. manufacturera: alimentos y bebidas - carnes",
            "Elaboración y conservación de pescado y productos de pescado": "Ind. manufacturera: alimentos y bebidas - pescado",
            "Elaboración de productos lácteos": "Ind. manufacturera: alimentos y bebidas - lácteos",
            "Elaboración de arroz y productos derivados del arroz": "Ind. manufacturera: alimentos y bebidas - arroz",
            "Elaboración de bebidas malteadas y de malta": "Ind. manufacturera: alimentos y bebidas - malta",
            "Elaboración de productos alimenticios y bebidas - Resto ": "Ind. manufacturera: alimentos y bebidas - otros",
            "Elaboración de Productos de Tabaco": "Ind. manufacturera: tabaco",
            "Fabricación de productos textiles": "Ind. manufacturera: textiles",
            "Preparación, hilandería, tejeduría y acabado de productos textiles": "Ind. manufacturera: textiles - hilandería y tejeduria",
            "Fabricación de prendas de vestir; adobo y  teñido de pieles": "Ind. manufacturera: prendas de vestir",
            "Curtido y adobo de cueros; fabricación de maletas, bolsos de mano, artículos de talabartería y guarnicionería, y calzado": "Ind. manufacturera: cueros",
            "Curtido y adobo de cueros; fabricación de maletas, bolsos de mano y artículos de talabartería y guarnicionería": "Ind. manufacturera: cueros - curtidos, adobos, derivados",
            "Producción de madera y fabricación de productos de madera y corcho excepto muebles; fabricación de artículos de paja y de materiales trenzables": "Ind. manufacturera: madera",
            "Fabricación de papel y de productos de papel; Actividades de edición e impresión y de reproducción de grabaciones": "Ind. manufacturera: papel",
            "Fabricación de coque, productos de la refinación del petróleo y combustible nuclear": "Ind. manufacturera: refinación de petróleo",
            "Fabricación de sustancias y productos químicos": "Ind. manufacturera: químicos",
            "Fabricación de sustancias químicas básicas, excepto abonos y compuestos de nitrógeno; fabricación de plásticos en formas primarias y de caucho sintético": "Ind. manufacturera: químicos - básicos excepto abonos",
            "Fabricación de abonos y compuestos de nitrógeno; Fabricación de plaguicidas y otros productos químicos de uso agropecuario": "Ind. manufacturera: químicos - abonos",
            "Fabricación de productos farmaceúticos, sustancias químicas medicinales y productos botánicos": "Ind. manufacturera: químicos - farmacéuticos",
            "Fabricación de jabones y detergentes, preparados para limpiar y pulir, perfumes y preparados de tocador": "Ind. manufacturera: químicos - jabones y detergentes",
            "Fabricación de productos de caucho y plástico": "Ind. manufacturera: caucho y plástico",
            "Fabricación de metales comunes; Fabricación de otros productos minerales no metálicos": "Ind. manufacturera: metales y minerales no metálicos",
            "Fabricación de vehículos automotores, remolques y semirremolques; Fabricación de otros tipos de equipo de transporte": "Ind. manufacturera: automotores",
            "INDUSTRIAS MANUFACTURERAS - Resto": "Ind. manufacturera: otros",
            "ELECTRICIDAD, GAS Y AGUA": "Electricidad, gas y agua: total",
        },
    },
    "trade_exports_sector_volume": {
        "start_col": 3,
        "currency": None,
        "unit": "2005=100",
        "colnames": {
            "TOTAL EXPORTACIONES ": "Total exportaciones",
            "AGRICULTURA, GANADERIA, CAZA Y SILVICULTURA": "Actividades primarias: total",
            "Trigo; servicios agrícolas aplicados al cultivo de trigo": "Actividades primarias: trigo",
            "Soja; girasol; servicios agrícolas aplicados al cultivo de soja y girasol": "Actividades primarias: soja y girasol",
            "Productos de árboles frutales, uvas, y plantas cuyas hojas o frutas se utilizan para preparar bebidas o especias; servicios agrícolas aplicados a estos cultivos": "Actividades primarias: prod. árboles frutales",
            "Productos de la cría de ganado vacuno, ovino, caprino, caballar, excepto producción de leche, lana y cueros; servicios ganaderos aplicados a esta actividad": "Actividades primarias: ganadería",
            "Madera y otros productos de la silvicultura. Servicios conexos": "Actividades primarias: silvicultura",
            "INDUSTRIAS MANUFACTURERAS": "Industrias manufactureras: total",
            "Carnes y productos del procesamiento y conservación de carne": "Ind. manufacturera: alimentos y bebidas - carnes",
            "Productos de la elaboración y conservación de pescado": "Ind. manufacturera: alimentos y bebidas - pescado",
            "Productos lácteos": "Ind. manufacturera: alimentos y bebidas - lácteos",
            "Arroz elaborado y otros productos derivados del arroz": "Ind. manufacturera: alimentos y bebidas - arroz",
            "Productos de la elaboración de cacao, chocolate, productos de confitería y otros productos alimenticios n.c.p.\n": "Ind. manufacturera: alimentos y bebidas - cacao",
            "Bebidas malteadas y malta": "Ind. manufacturera: alimentos y bebidas - malta",
            "Productos de lavaderos; hilados y tejidos; otros productos de la hilandería, tejeduría y acabado de productos textiles": "Ind. manufacturera: textiles",
            "Prendas de vestir; adobo y teñido de pieles": "Ind. manufacturera: prendas de vestir",
            "Cueros elaborados; artículos de talabartería y guarnicionería": "Ind. manufacturera: cueros - curtidos, adobos, derivados",
            "Productos de aserrado y otros artículos de madera, excepto muebles": "Ind. manufacturera: madera",
            "Papel, cartón y subproductos": "Ind. manufacturera: papel",
            "Productos de la refinación del petróleo y combustible nuclear": "Ind. manufacturera: refinación de petróleo",
            "Abonos y compuestos de nitrógeno": "Ind. manufacturera: químicos - abonos",
            "Sustancias químicas básicas, excepto abonos": "Ind. manufacturera: químicos - básicos excepto abonos",
            "Productos farmacéuticos, sustancias químicas medicinales y productos botánicos, de uso humano y animal": "Ind. manufacturera: químicos - farmacéuticos",
            "Productos de limpieza y tocador, primarias y de caucho sintético": "Ind. manufacturera: químicos - jabones y detergentes",
            "Productos de caucho y plástico": "Ind. manufacturera: caucho y plástico",
            "Productos de fundición de hierro y acero; aluminio y otros metales no ferrosos": "Ind. manufacturera: metales y minerales no metálicos",
            "Vehículos automotores, remolques, semiremolques y contenedores; partes y piezas elaboradas para los mismos": "Ind. manufacturera: automotores",
        },
    },
    "trade_exports_sector_price": {
        "start_col": 3,
        "currency": "USD",
        "unit": "2005=100",
        "colnames": {
            "TOTAL EXPORTACIONES ": "Total exportaciones",
            "AGRICULTURA, GANADERIA, CAZA Y SILVICULTURA": "Actividades primarias: total",
            "Trigo; servicios agrícolas aplicados al cultivo de trigo": "Actividades primarias: trigo",
            "Soja; girasol; servicios agrícolas aplicados al cultivo de soja y girasol": "Actividades primarias: soja y girasol",
            "Productos de árboles frutales, uvas, y plantas cuyas hojas o frutas se utilizan para preparar bebidas o especias; servicios agrícolas aplicados a estos cultivos": "Actividades primarias: prod. árboles frutales",
            "Productos de la cría de ganado vacuno, ovino, caprino, caballar, excepto producción de leche, lana y cueros; servicios ganaderos aplicados a esta actividad": "Actividades primarias: ganadería",
            "Madera y otros productos de la silvicultura. Servicios conexos": "Actividades primarias: silvicultura",
            "INDUSTRIAS MANUFACTURERAS": "Industrias manufactureras: total",
            "Carnes y productos del procesamiento y conservación de carne": "Ind. manufacturera: alimentos y bebidas - carnes",
            "Productos de la elaboración y conservación de pescado": "Ind. manufacturera: alimentos y bebidas - pescado",
            "Productos lácteos": "Ind. manufacturera: alimentos y bebidas - lácteos",
            "Arroz elaborado y otros productos derivados del arroz": "Ind. manufacturera: alimentos y bebidas - arroz",
            "Productos de la elaboración de cacao, chocolate, productos de confitería y otros productos alimenticios n.c.p.": "Ind. manufacturera: alimentos y bebidas - cacao",
            "Bebidas malteadas y malta": "Ind. manufacturera: alimentos y bebidas - malta",
            "Productos de lavaderos; hilados y tejidos; otros productos de la hilandería, tejeduría y acabado de productos textiles": "Ind. manufacturera: textiles",
            "Prendas de vestir; adobo y teñido de pieles": "Ind. manufacturera: prendas de vestir",
            "Cueros elaborados; artículos de talabartería y guarnicionería": "Ind. manufacturera: cueros - curtidos, adobos, derivados",
            "Productos de aserrado y otros artículos de madera, excepto muebles": "Ind. manufacturera: madera",
            "Papel, cartón y subproductos": "Ind. manufacturera: papel",
            "Productos de la refinación del petróleo y combustible nuclear": "Ind. manufacturera: refinación de petróleo",
            "Abonos y compuestos de nitrógeno": "Ind. manufacturera: químicos - abonos",
            "Sustancias químicas básicas, excepto abonos": "Ind. manufacturera: químicos - básicos excepto abonos",
            "Productos farmacéuticos, sustancias químicas medicinales y productos botánicos, de uso humano y animal": "Ind. manufacturera: químicos - farmacéuticos",
            "Productos de limpieza y tocador, primarias y de caucho sintético": "Ind. manufacturera: químicos - jabones y detergentes",
            "Productos de caucho y plástico": "Ind. manufacturera: caucho y plástico",
            "Productos de fundición de hierro y acero; aluminio y otros metales no ferrosos": "Ind. manufacturera: metales y minerales no metálicos",
            "Vehículos automotores, remolques, semiremolques y contenedores; partes y piezas elaboradas para los mismos": "Ind. manufacturera: automotores",
        },
    },
    "trade_exports_destination_value": {
        "start_col": 1,
        "currency": "USD",
        "unit": "Millions",
        "colnames": {
            "TOTAL EXPORTACIONES": "Total exportaciones",
            "AMERICA": "América: total",
            "ALADI 2/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "Sub-total MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "Sub-total MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Bolivia": "América: Bolivia",
            "Chile": "América: Chile",
            "México": "América: México",
            "Otros de Aladi": "América: otros ALADI",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "RESTO DE AMERICA": "América: otros",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "Otros de la U.E.": "Europa: UE - otros",
            "AELC (EFTA)1/": "Europa: EFTA",
            "RESTO DE EUROPA": "Europa: otros",
            "Rusia": "Europa: otros - Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
            "AFRICA": "Resto del mundo: África",
            "ASIA (excl. M. Oriente)": "Resto del mundo: Asia - total",
            "China (Continental)": "Resto del mundo: Asia - China",
            "Corea del Sur": "Resto del mundo: Asia - Corea del Sur",
            "Taiwán": "Resto del mundo: Asia - Taiwán",
            "Hong Kong": "Resto del mundo: Asia - Hong Kong",
            "Japón": "Resto del mundo: Asia - Japón",
            "Otros de Asia": "Resto del mundo: Asia - otros",
            "MEDIO ORIENTE": "Resto del mundo: Medio Oriente - total",
            "Arabia": "Resto del mundo: Medio Oriente - Arabia",
            "Irán": "Resto del mundo: Medio Oriente - Irán",
            "Israel": "Resto del mundo: Medio Oriente - Israel",
            "Otros del Medio Oriente": "Resto del mundo: Medio Oriente - otros",
            "OTROS PAISES": "Resto del mundo: otros",
        },
    },
    "trade_exports_destination_volume": {
        "start_col": 1,
        "currency": None,
        "unit": "2005=100",
        "colnames": {
            "TOTAL EXPORTACIONES": "Total exportaciones",
            "AMERICA": "América: total",
            "ALADI 1/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "SUB-TOTAL  MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "SUB-TOTAL  MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Chile": "América: Chile",
            "México": "América: México",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "RESTO DE EUROPA": "Europa: otros",
            "Rusia": "Europa: otros - Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
        },
    },
    "trade_exports_destination_price": {
        "start_col": 1,
        "currency": "USD",
        "unit": "2005=100",
        "colnames": {
            "TOTAL EXPORTACIONES": "Total exportaciones",
            "AMERICA": "América: total",
            "ALADI 1/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "Sub-total MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "SUB-TOTAL  MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Chile": "América: Chile",
            "México": "América: México",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "RESTO DE EUROPA": "Europa: otros",
            "Rusia": "Europa: otros - Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
        },
    },
    "trade_imports_category_value": {
        "start_col": 2,
        "currency": "USD",
        "unit": "Millions",
        "old_colnames": [
            "TOTAL IMPORTACIONES FOB ",
            "TOTAL IMPORTACIONES CIF ",
            "TOTAL IMPORTACIONES CIF SIN PETRÓLEO, DESTILADOS Y ENERGÍA ELÉCTRICA",
            "CONSUMO",
            "Alimentos y bebidas",
            "Automotores y otros de transporte",
            "Duraderos",
            "Otros de consumo",
            "CAPITAL",
            "Maquinaria y equipos",
            "Público",
            "Privado",
            "Equipos de transporte",
            "Público",
            "Privado",
            "INTERMEDIOS ",
            "Petróleo y destilados ",
            "Energía eléctrica",
            "Intermedios sin petróleo, destilados y energía eléctrica",
            "Alimentos y bebidas",
            "Piezas y accesorios de transporte",
            "Otros insumos intermedios",
            "Suministros industriales básicos",
            "Suministros industriales elaborados",
            "Resto",
        ],
        "new_colnames": [
            "Total importaciones: FOB",
            "Total importaciones: CIF ",
            "Total importaciones: CIF sin petróleo y energía eléct.",
            "Consumo: total",
            "Consumo: alimentos y bebidas",
            "Consumo: automotores",
            "Consumo: duraderos",
            "Consumo: otros",
            "Capital: total",
            "Capital: maquinaria y equipos",
            "Capital: maquinaria y equipos - público",
            "Capital: maquinaria y equipos - privado",
            "Capital: equipos de transporte",
            "Capital: equipos de transporte - público",
            "Capital: equipos de transporte - privado",
            "Intermedios: total",
            "Intermedios: petróleo y destilados",
            "Intermedios: energía eléctrica",
            "Intermedios: total sin petróleo y energía eléctrica",
            "Intermedios: alimentos y bebidas",
            "Intermedios: piezas de transporte",
            "Intermedios: otros",
            "Intermedios: otros - industriales básicos",
            "Intermedios: otros - industriales elaborados",
            "Intermedios: otros - resto",
        ],
    },
    "trade_imports_category_volume": {
        "start_col": 1,
        "currency": None,
        "unit": "2005=100",
        "colnames": {
            "TOTAL IMPORTACIONES": "Total importaciones",
            "CONSUMO": "Consumo: total",
            "ALIMENTOS Y BEBIDAS PARA LOS HOGARES": "Consumo: alimentos y bebidas",
            "AUTOMOTORES Y OTROS DE TRANSPORTE": "Consumo: automotores",
            "CONSUMO DURADERO": "Consumo: duraderos",
            "OTROS DE CONSUMO": "Consumo: otros",
            "CAPITAL": "Capital: total",
            "INTERMEDIOS": "Intermedios: total",
            "PETROLEO Y DESTILADOS": "Intermedios: petróleo",
            "INTERMEDIOS SIN PETROLEO, DESTILADOS Y ENERGIA ELECTRICA": "Intermedios: total sin petróleo y energía eléctrica",
            "ALIMENTOS Y BEBIDAS PARA LA INDUSTRIA": "Intermedios: alimentos y bebidas",
            "PIEZAS Y ACCESORIOS DE TRANSPORTE": "Intermedios: transporte",
            "OTROS INSUMOS INTERMEDIOS": "Intermedios: otros",
        },
    },
    "trade_imports_category_price": {
        "start_col": 1,
        "currency": "USD",
        "unit": "2005=100",
        "colnames": {
            "TOTAL IMPORTACIONES": "Total importaciones",
            "CONSUMO": "Consumo: total",
            "ALIMENTOS Y BEBIDAS PARA LOS HOGARES": "Consumo: alimentos y bebidas",
            "AUTOMOTORES Y OTROS DE TRANSPORTE": "Consumo: automotores",
            "CONSUMO DURADERO": "Consumo: duraderos",
            "OTROS DE CONSUMO": "Consumo: otros",
            "CAPITAL": "Capital: total",
            "INTERMEDIOS": "Intermedios: total",
            "PETROLEO Y DESTILADOS": "Intermedios: petróleo",
            "INTERMEDIOS SIN PETROLEO, DESTILADOS Y ENERGIA ELECTRICA": "Intermedios: total sin petróleo y energía eléctrica",
            "ALIMENTOS Y BEBIDAS PARA LA INDUSTRIA": "Intermedios: alimentos y bebidas",
            "PIEZAS Y ACCESORIOS DE TRANSPORTE": "Intermedios: transporte",
            "OTROS INSUMOS INTERMEDIOS": "Intermedios: otros",
        },
    },
    "trade_imports_origin_value": {
        "start_col": 1,
        "currency": "USD",
        "unit": "Millions",
        "colnames": {
            "TOTAL IMPORTACIONES CIF ": "Total importaciones",
            "AMERICA": "América: total",
            "ALADI 2/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "Sub-total MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "Sub-total MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Bolivia": "América: Bolivia",
            "Chile": "América: Chile",
            "México": "América: México",
            "Otros de Aladi": "América: otros ALADI",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "RESTO DE AMERICA": "América: otros",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "Otros de la U.E.": "Europa: UE - otros",
            "AELC (EFTA)1/": "Europa: EFTA",
            "RESTO DE EUROPA": "Europa: otros",
            "Rusia": "Europa: otros - Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
            "AFRICA": "Resto del mundo: África",
            "ASIA (excl. M. Oriente)": "Resto del mundo: Asia - total",
            "China (Continental)": "Resto del mundo: Asia - China",
            "Corea del Sur": "Resto del mundo: Asia - Corea del Sur",
            "Taiwán": "Resto del mundo: Asia - Taiwán",
            "Hong Kong": "Resto del mundo: Asia - Hong Kong",
            "Japón": "Resto del mundo: Asia - Japón",
            "Otros de Asia": "Resto del mundo: Asia - otros",
            "MEDIO ORIENTE": "Resto del mundo: Medio Oriente - total",
            "Arabia": "Resto del mundo: Medio Oriente - Arabia",
            "Irán": "Resto del mundo: Medio Oriente - Irán",
            "Israel": "Resto del mundo: Medio Oriente - Israel",
            "Otros del Medio Oriente": "Resto del mundo: Medio Oriente - otros",
            "OTROS PAISES": "Resto del mundo: otros",
        },
    },
    "trade_imports_origin_volume": {
        "start_col": 1,
        "currency": None,
        "unit": "2005=100",
        "colnames": {
            "TOTAL IMPORTACIONES": "Total importaciones",
            "AMERICA": "América: total",
            "ALADI 1/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "Sub-total MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "SUB-TOTAL  MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Chile": "América: Chile",
            "México": "América: México",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "RUSIA": "Europa: Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
            "China (Continental)": "Resto del mundo: China",
        },
    },
    "trade_imports_origin_price": {
        "start_col": 1,
        "currency": "USD",
        "unit": "2005=100",
        "colnames": {
            "TOTAL IMPORTACIONES": "Total importaciones",
            "AMERICA": "América: total",
            "ALADI 1/": "América: ALADI",
            "Argentina": "América: Argentina",
            "Brasil": "América: Brasil",
            "Paraguay": "América: Paraguay",
            "Sub-total MERCOSUR (4 PAISES)": "América: MERCOSUR 4 países",
            "Venezuela": "América: Venezuela",
            "SUB-TOTAL  MERCOSUR (5 PAISES)": "América: MERCOSUR 5 países",
            "Chile": "América: Chile",
            "México": "América: México",
            "CANADA": "América: Canadá",
            "ESTADOS UNIDOS": "América: Estados Unidos",
            "EUROPA": "Europa: total",
            "U.E. (UNION EUROPEA)": "Europa: Unión Europea - total",
            "Alemania": "Europa: UE - Alemania",
            "España": "Europa: UE - España",
            "Francia": "Europa: UE - Francia",
            "Italia": "Europa: UE - Italia",
            "Países Bajos": "Europa: UE - Países Bajos",
            "Reino Unido": "Europa: UE - Reino Unido",
            "RUSIA": "Europa: Rusia",
            "RESTO DEL MUNDO": "Resto del mundo: total",
            "China (Continental)": "Resto del mundo: China",
        },
    },
}
FISCAL_SHEETS = {
    "fiscal_balance_nonfinancial_public_sector": {
        "sheet": "Sector Público No Monetario",
        "colnames": [
            "Ingresos: SPNF",
            "Ingresos: Gobierno central",
            "Ingresos: DGI",
            "Ingresos: IRP",
            "Ingresos: Comercio ext.",
            "Ingresos: Otros",
            "Ingresos: BPS",
            "Ingresos: Res. primario corriente EEPP",
            "Egresos: Primarios SPNF",
            "Egresos: Primarios corrientes GC-BPS",
            "Egresos: Remuneraciones",
            "Egresos: No personales",
            "Egresos: Pasividades",
            "Egresos: Transferencias",
            "Egresos: Inversiones",
            "Resultado: Primario intendencias",
            "Resultado: Primario BSE",
            "Resultado: Primario SPNF",
            "Intereses: Totales",
            "Intereses: GC-BPS",
            "Intereses: EEPP",
            "Intereses: Intendencias",
            "Intereses: BSE",
            "Resultado: Global SPNF",
        ],
    },
    "fiscal_balance_global_public_sector": {
        "sheet": "Sector Público Consolidado",
        "colnames": [
            "Resultado: Primario SPNF",
            "Intereses: SPNF",
            "Resultado: Global SPNF",
            "Resultado: Primario BCU",
            "Intereses: BCU",
            "Resultado: Global BCU",
            "Resultado: Primario SPC",
            "Resultado: Global SPC",
        ],
    },
    "fiscal_balance_central_government": {
        "sheet": "Gobierno Central - BPS",
        "colnames": [
            "Ingresos: GC-BPS",
            "Ingresos: GC",
            "Ingresos: Comercio ext.",
            "Ingresos: DGI",
            "Ingresos: DGI bruto",
            "Ingresos: DGI CDI",
            "Ingresos: Loterías",
            "Ingresos: Venta energía",
            "Ingresos: TGN/otros",
            "Ingresos: FIMTOP",
            "Ingresos: Aportes EEPP",
            "Ingresos: IRP",
            "Ingresos: Rec. Lib. Disp",
            "Ingresos: BPS neto",
            "Ingresos: BPS bruto",
            "Ingresos: FSS - Cincuentones",
            "Ingresos: BPS CDI",
            "Ingresos: BPS otros",
            "Egresos: GC-BPS",
            "Egresos: Remuneraciones",
            "Egresos: Remuneraciones adm. central",
            "Egresos: Remuneraciones org. docentes",
            "Egresos: Remuneraciones retenc./otros",
            "Egresos: Remuneraciones BPS",
            "Egresos: Pasividades",
            "Egresos: Pasividades Caja Policial",
            "Egresos: Pasividades Caja Militar",
            "Egresos: Pasividades BPS",
            "Egresos: No personales",
            "Egresos: No personales adm. central",
            "Egresos: No personales org. docentes",
            "Egresos: No personales suministros",
            "Egresos: No personales plan emergencia",
            "Egresos: No personales BPS",
            "Egresos: Transferencias",
            "Egresos: Transferencias GC",
            "Egresos: Transferencias GC Entes",
            "Egresos: Transferencias GC deuda",
            "Egresos: Transferencias GC otros org.",
            "Egresos: Transferencias GC rentas afectadas",
            "Egresos: Transferencias BPS",
            "Egresos: Transferencias BPS enfermedad",
            "Egresos: Transferencias BPS AFAM y otras prestaciones",
            "Egresos: Transferencias BPS desempleo",
            "Egresos: Transferencias BPS 2",
            "Egresos: Transferencias BPS -IRP/IRPF",
            "Egresos: Transferencias BPS AFAP",
            "Egresos: Transferencias BPS otros",
            "Egresos: Transferencias otros",
            "Egresos: Inversión",
            "Egresos: Inversión MTOP",
            "Egresos: Inversión MVOTMA",
            "Egresos: Inversión Presidencia",
            "Egresos: Inversión org. docentes",
            "Egresos: Inversión resto",
            "Intereses: Total",
            "Intereses: GC",
            "Intereses: FSS - Cincuentones",
            "Resultado: Global GC-BPS",
        ],
    },
    "fiscal_balance_soe": {
        "sheet": "Empresas Públicas Consolidado",
        "colnames": [
            "Ingresos",
            "Ingresos: Venta bienes y servicios",
            "Ingresos: Otros",
            "Ingresos: Transferencias GC",
            "Egresos",
            "Egresos: Corrientes",
            "Egresos: Remuneraciones",
            "Egresos: Compras bienes y servicios",
            "Intereses",
            "Egresos: DGI",
            "Egresos: BPS",
            "Egresos: No corrientes",
            "Egresos: Inversiones",
            "Egresos: Dividendo",
            "Resultado: Global",
        ],
    },
    "fiscal_balance_ancap": {
        "sheet": "ANCAP",
        "colnames": [
            "Ingresos",
            "Ingresos: Venta bienes y servicios",
            "Ingresos: Otros",
            "Ingresos: Transferencias GC",
            "Egresos",
            "Egresos: Corrientes",
            "Egresos: Remuneraciones",
            "Egresos: Compras bienes y servicios",
            "Intereses",
            "Egresos: DGI",
            "Egresos: BPS",
            "Egresos: No corrientes",
            "Egresos: Inversiones",
            "Egresos: Var. stock petróleo",
            "Egresos: Otros",
            "Egresos: Dividendo",
            "Resultado: Global",
        ],
    },
    "fiscal_balance_antel": {
        "sheet": "ANTEL",
        "colnames": [
            "Ingresos",
            "Ingresos: Venta bienes y servicios",
            "Ingresos: Otros",
            "Ingresos: Transferencias GC",
            "Egresos",
            "Egresos: Corrientes",
            "Egresos: Remuneraciones",
            "Egresos: Compras bienes y servicios",
            "Intereses",
            "Egresos: DGI",
            "Egresos: BPS",
            "Egresos: No corrientes",
            "Egresos: Inversiones",
            "Egresos: Dividendo",
            "Resultado: Global",
        ],
    },
    "fiscal_balance_ose": {
        "sheet": "OSE",
        "colnames": [
            "Ingresos",
            "Ingresos: Venta bienes y servicios",
            "Ingresos: Otros",
            "Ingresos: Transferencias GC",
            "Egresos",
            "Egresos: Corrientes",
            "Egresos: Remuneraciones",
            "Egresos: Compras bienes y servicios",
            "Intereses",
            "Egresos: DGI",
            "Egresos: BPS",
            "Egresos: No corrientes",
            "Egresos: Inversiones",
            "Egresos: Dividendo",
            "Resultado: Global",
        ],
    },
    "fiscal_balance_ute": {
        "sheet": "UTE",
        "colnames": [
            "Ingresos",
            "Ingresos: Venta bienes y servicios",
            "Ingresos: Otros",
            "Ingresos: Transferencias GC",
            "Egresos",
            "Egresos: Corrientes",
            "Egresos: Remuneraciones",
            "Egresos: Compras bienes y servicios",
            "Intereses",
            "Egresos: DGI",
            "Egresos: BPS",
            "Egresos: No corrientes",
            "Egresos: Inversiones",
            "Egresos: Dividendo",
            "Resultado: Global",
        ],
    },
}
RESERVES_COLUMNS = [
    "Saldo al inicio del período",
    "1. Compras netas de moneda extranjera",
    "1.1. Compras netas en el mercado",
    "1.2. Integración en dólares de valores del BCU",
    "1.3. Prefinanciación de exportaciones",
    "1.4. Cancelación de contratos forward",
    "1.5. Gobierno Central",
    "1.5.1. Utilizaciones de préstamos internacionales",
    "1.5.2. Otras compras netas al Gobierno Central",
    "1.6. Otros",
    "2. Depósitos del Sistema Bancario en el Banco Central",
    "2.1. Banca pública",
    "2.2. Banca privada",
    "3. Otros Depósitos en el Banco Central",
    "3.1. Depósitos de otras empresas de intermediación fin.",
    "3.2. Depósitos de casas de cambio y otras inst.",
    "3.3. Depósitos de empresas públicas y gob. dptales.",
    "4. Divisas de exportación a liquidar",
    "5. Obligaciones netas en ME con gob. central",
    "5.1. Colocación neta de bonos y letras",
    "5.1.1. Colocación bruta",
    "5.1.2. Amortizaciones",
    "5.1.3. Intereses y comisiones",
    "5.2. Otras obligaciones netas en ME",
    "5.2.1. Desembolsos de préstamos internacionales",
    "5.2.2. Servicio neto de préstamos internacionales",
    "5.3.3. Utilizaciones de préstamos internacionales",
    "5.4.5. Aporte de entes a cuenta de resultados",
    "5.4.6.  Compras de moneda extranjera",
    "5.4.7. Giros hacia y desde el BROU",
    "5.4.8. Integración en USD de títulos en UI y pesos-neto",
    "5.4.9. Otros",
    "6. Intereses netos",
    "6.1.Intereses pagados sobre depósitos del sist. fin.",
    "6.2. Intereses cobrados sobre fondos en el exterior",
    "6.3. Otros intereses y comisiones netos",
    "7. Otros",
    "7.1.  Préstamos y financiamientos EEPP",
    "7.2. Cuentas con organismos internacionales",
    "7.3. Fondos administrados",
    "7.4. Diferencias de arbitraje",
    "7.5. Diferencias de cotización e intereses devengados",
    "7.6. Depósitos especiales Clearstream Banking",
    "7.7. Solicitudes de giro al exterior en trámite",
    "7.8. Otros",
    "Variación total del período - 1+2+3+4+5+6+7",
    "Saldo al final del período",
]

cpi_details = {
    "2010_base": {
        "Pan y cereales": {"Tradable": True, "Core": True},
        "Carne": {"Tradable": True, "Core": False},
        "Pescado": {"Tradable": True, "Core": True},
        "Leche, queso y huevos": {"Tradable": True, "Core": True},
        "Aceites y grasas": {"Tradable": True, "Core": True},
        "Frutas": {"Tradable": True, "Core": False},
        "Legumbres y Hortalizas": {"Tradable": True, "Core": False},
        "Azúcar, mermelada, miel, chocolate y dulces de azúcar": {
            "Tradable": True,
            "Core": True,
        },
        "Productos alimenticios n.e.p.": {"Tradable": True, "Core": True},
        "Bebidas no alcohólicas": {"Tradable": True, "Core": True},
        "Bebidas alcohólicas": {"Tradable": True, "Core": True},
        "Tabaco": {"Tradable": True, "Core": False},
        "PRENDAS DE VESTIR Y CALZADO": {"Tradable": True, "Core": True},
        "Alquileres efectivos del alojamiento": {"Tradable": False, "Core": True},
        "Conservación y reparación de la vivienda": {"Tradable": False, "Core": True},
        "Suministro de agua y servicios diversos relacionados con la vivienda": {
            "Tradable": False,
            "Core": False,
        },
        "Electricidad, gas y otros combustibles": {"Tradable": False, "Core": False},
        "Muebles y accesorios, alfombras y otros materiales para pisos": {
            "Tradable": True,
            "Core": True,
        },
        "Productos textiles para el hogar": {"Tradable": True, "Core": True},
        "Artefactos para el hogar grandes, eléctricos o no": {
            "Tradable": True,
            "Core": True,
        },
        "Reparación de artefactos para el hogar": {"Tradable": False, "Core": True},
        "Artículos de vidrio y cristal, vajilla y utensilios para el hogar": {
            "Tradable": True,
            "Core": True,
        },
        "Herramientas y equipo para el hogar y el jardín": {
            "Tradable": True,
            "Core": True,
        },
        "Bienes para el hogar no duraderos": {"Tradable": True, "Core": True},
        "Servicios domésticos y para el hogar": {"Tradable": False, "Core": True},
        "Productos, artefactos y equipo médicos": {"Tradable": True, "Core": True},
        "Servicios para pacientes externos": {"Tradable": False, "Core": False},
        "Servicios de hospital": {"Tradable": False, "Core": False},
        "Servicios médicos mutuales y colectivos": {"Tradable": False, "Core": False},
        "Adquisición de vehículos": {"Tradable": True, "Core": True},
        "Funcionamiento de equipo de transporte personal": {
            "Tradable": False,
            "Core": True,
        },
        "Servicios de transporte": {"Tradable": False, "Core": False},
        "Servicios postales": {"Tradable": False, "Core": False},
        "Equipo telefónico y de facsímile": {"Tradable": True, "Core": True},
        "Servicios telefónicos y de facsímile": {"Tradable": False, "Core": False},
        "Equipo audiovisual, fotográfico y de procesamiento de información": {
            "Tradable": True,
            "Core": True,
        },
        "Otros artículos y equipo para recreación, jardines y animales domésticos": {
            "Tradable": True,
            "Core": True,
        },
        "Servicios de recreación y culturales": {"Tradable": False, "Core": False},
        "Periódicos, libros y papeles y útiles de oficina": {
            "Tradable": True,
            "Core": True,
        },
        "Paquetes turísticos": {"Tradable": True, "Core": True},
        "EDUCACIÓN": {"Tradable": False, "Core": True},
        "Servicios de alojamiento": {"Tradable": False, "Core": True},
        "Servicios de suministro de comidas por contrato": {
            "Tradable": False,
            "Core": True,
        },
        "Cuidado personal": {"Tradable": False, "Core": True},
        "Efectos personales n.e.p.": {"Tradable": False, "Core": True},
        "Seguros": {"Tradable": False, "Core": True},
        "Otros servicios n.e.p.": {"Tradable": False, "Core": True},
    },
    "1997_base": {
        "    Panes Y Cereales": {"Tradable": True, "Core": True},
        "    Carnes Y Derivados": {"Tradable": True, "Core": False},
        "    Lacteos Y Huevos": {"Tradable": True, "Core": True},
        "    Aceites Y Grasas": {"Tradable": True, "Core": True},
        "    Frutas": {"Tradable": True, "Core": False},
        "    Verduras, Legumbres Y Tuberculos": {"Tradable": True, "Core": False},
        "    Azucar, Cafe, Te, Yerba Y Cacao": {"Tradable": True, "Core": True},
        "    Otros Alimentos": {"Tradable": True, "Core": True},
        "    Bebidas No Alcoholicas": {"Tradable": True, "Core": True},
        "    Bebidas Alcoholicas": {"Tradable": True, "Core": True},
        "  Comidas Semielaboradas, Elaboradas": {"Tradable": True, "Core": True},
        "Vestimenta Y Calzado": {"Tradable": True, "Core": True},
        "  Alquileres, Mantenimiento, Gastos Comunes": {
            "Tradable": False,
            "Core": True,
        },
        "  Combustible, Alumbrado Y Agua Corriente": {"Tradable": False, "Core": False},
        "  Muebles, Acc. Fijos Y Reparaciones": {"Tradable": True, "Core": True},
        "  Tejidos Para El Hogar Y Otros Accesorios": {"Tradable": True, "Core": True},
        "  Artefactos, Electrodomesticos, Cristaleria": {
            "Tradable": True,
            "Core": True,
        },
        "  Servicios Para El Hogar Exc. Servicio Domestico": {
            "Tradable": False,
            "Core": True,
        },
        "  Servicio Domestico Y Reparaciones ": {"Tradable": False, "Core": True},
        "  Productos Medicinales Y Farmaceuticos": {"Tradable": True, "Core": True},
        "  Aparatos Y Equipos Terapeuticos": {"Tradable": True, "Core": True},
        "  Servicios Medicos": {"Tradable": False, "Core": False},
        "  Equipo De Transporte Personal": {"Tradable": True, "Core": True},
        "  Mantenimiento Y Utilizacion De Equipo Tpte.": {
            "Tradable": False,
            "Core": True,
        },
        "  Servicios De Transporte": {"Tradable": False, "Core": False},
        "  Comunicaciones": {"Tradable": False, "Core": False},
        "  Equipos Accesorios Y Reparaciones": {"Tradable": True, "Core": True},
        "  Servicios De Esparcimiento": {"Tradable": False, "Core": True},
        "  Libros, Periodicos Y Revistas": {"Tradable": True, "Core": True},
        "  Turismo Y Alojamiento": {"Tradable": True, "Core": True},
        "Enseñanza": {"Tradable": False, "Core": False},
        "  Cuidados Y Efectos Personales": {"Tradable": False, "Core": True},
        "  Tabaco": {"Tradable": True, "Core": False},
        "  Servicios Diversos": {"Tradable": False, "Core": True},
    },
    "1997_weights": [
        "PAN Y CEREALES",
        "CARNES Y DERIVADOS",
        "LÁCTEOS Y HUEVOS",
        "ACEITES Y GRASAS",
        "FRUTAS",
        "VERDURAS, LEGUMBRES Y TUBÉRCULOS",
        "AZÚCAR, CAFÉ, TÉ, YERBA MATE Y CACAO",
        "OTROS ALIMENTOS",
        "BEBIDAS NO ALCOHÓLICAS",
        "BEBIDAS ALCOHÓLICAS",
        "COMIDAS SEMIELABORADAS, ELABORADAS Y FUERA DEL HOGAR",
        "VESTIMENTA Y CALZADO",
        "ALQUILER, MANTENIMIENTO, GASTOS COMUNES E IMPUESTOS",
        "COMBUSTIBLE, ALUMBRADO Y AGUA CORRIENTE",
        "MUEBLES, ACCESORIOS FIJOS Y REPARACIONES",
        "TEJIDOS PARA EL HOGAR Y OTROS ACCESORIOS",
        "ELECTRODOMÉSTICOS, CRISTALERÍA, VAJILLA Y UTENSILIOS DOMÉSTICOS",
        "SERVICIOS PARA EL HOGAR (EXCEPTO SERVICIO DOMÉSTICO)",
        "SERVICIO DOMÉSTICO Y REPARACIONES DEL HOGAR",
        "PRODUCTOS MEDICINALES Y FARMACÉUTICOS",
        "APARATOS Y EQUIPOS TERAPÉUTICOS",
        "SERVICIOS MÉDICOS",
        "EQUIPO DE TRANSPORTE PERSONAL",
        "MANTENIMIENTO Y UTILIZACIÓN DE EQUIPO DE TRANSPORTE PERSONAL",
        "SERVICIOS DE TRANSPORTE",
        "COMUNICACIONES",
        "EQUIPOS, ACCESORIOS Y REPARACIONES",
        "SERVICIO DE ESPARCIMIENTO",
        "LIBROS, PERIÓDICOS Y REVISTAS",
        "TURISMO Y ALOJAMIENTO",
        "ENSEÑANZA",
        "CUIDADOS Y EFECTOS PERSONALES",
        "TABACO",
        "SERVICIOS DIVERSOS",
    ],
}
taxes_columns = [
    "IRAE - Rentas de Actividades Económicas",
    "IMEBA - Enajenación de Bienes Agropecuarios",
    "IRPF Cat I - Renta de las Personas Físicas",
    "IRPF Cat II - Rentas de las Personas Físicas",
    "IRNR - Rentas de No Residentes",
    "IASS - Asistencia a la Seguridad Social",
    "ISAFI - SA Financieras de Inversión",
    "IP Personas Jurídicas - Patrimonio",
    "IP Explotaciones Agropecuarias - Patrimonio",
    "IP Sobretasa - IP",
    "ICOSA - Control de las SA",
    "IP Personas Físicas - Patrimonio",
    "ITP - Transferencias Patrimoniales",
    "ITP Adicional - Transferencias Patrimoniales",
    "IVA - Valor Agregado",
    "IVA Mínimo - Pequeña Empresa",
    "IMESI - Específico Interno",
    "IMEBA Ad. MEVIR - Enajernación de Bienes Agropecuarios",
    "IMEBA Ad. INIA - Enajernación de Bienes Agropecuarios",
    "Impuesto a los Ingresos de las Compañías de Seguros",
    "FIS - Fondo de Inspección Sanitaria",
    "IRIC - Rentas de Industria Comercio",
    "COFIS - Contribución al Financiamiento Seguridad Social",
    "IMESSA - Específico a los Servicios de Salud",
    "Impuesto a las Comisiones",
    "Impuesto a las Ventas Forzadas",
    "Impuesto a las Tarjetas de Crédito",
    "ITEL - Telecomunicaciones",
    "Impuesto a los Ingresos le Los Organizadores de Sorteos",
    "ICOME - Compra de Moneda Extranjera",
    "Impuesto a la Compra - Venta de Bienes Remate Público",
    "Impuesto a las Cesiones de Derechos sobre Deportistas",
    "IMABA - Activos de las Empresas Bancarias",
    "ICOSIFI - Control del Sistema Financiero",
    "IRA - Rentas Agropecuarias",
    "IMAGRO - Actividades Agropecuaria",
    "Impuesto de Educación Primaria",
    "Recaudación Total de la DGI",
]


BOP_COLUMNS = [
    "Cuenta Corriente",
    "Bienes y Servicios",
    "Bienes",
    "Bienes - Crédito",
    "Bienes - Crédito - Mercancías generales",
    "Bienes - Crédito - Mercancías generales - Compraventa",
    "Bienes - Crédito - Exp netas de bienes en compra venta",
    "Bienes - Crédito - Exp netas - Compras por Compraventa",
    "Bienes - Crédito - Exp netas - Ventas por compraventa",
    "Bienes - Crédito - Exp netas - Soc. Compraventa",
    "Bienes - Débito",
    "Bienes - Débito - Mercancías generales",
    "Bienes - Débito - Mercancías generales - Compraventa",
    "Servicios",
    "Servicios - Crédito",
    "Servicios - Crédito - Transportes",
    "Servicios - Crédito - Viajes",
    "Servicios - Crédito - Otros Servicios",
    "Servicios - Crédito - Otros - Compraventa",
    "Servicios - Débito",
    "Servicios - Débito - Transportes",
    "Servicios - Débito - Viajes",
    "Servicios - Débito - Otros Servicios",
    "Servicios - Débito - Otros -  Compraventa",
    "Ingreso Primario",
    "Ingreso Primario - Crédito",
    "Ingreso Primario - Crédito - Institucional, Sector Público",
    "Ingreso Primario - Crédito - Institucional, Sector Privado",
    "Ingreso Primario - Crédito - Inst, Priv - Soc. Compraventa",
    "Ingreso Primario - Crédito - Funcional, Inversión Directa",
    "Ingreso Primario - Crédito - Funcional, Inversión de Cartera",
    "Ingreso Primario - Crédito - Funcional, Otra Inversión",
    "Ingreso Primario - Crédito - Funcional, Activos de Reserva",
    "Ingreso Primario - Débito",
    "Ingreso Primario - Débito - Institucional, Sector Público",
    "Ingreso Primario - Débito - Institucional, Sector Privado",
    "Ingreso Primario - Débito - Inst, Priv - Soc. Compraventa",
    "Ingreso Primario - Débito - Funcional, Inversión Directa",
    "Ingreso Primario - Débito - Funcional, Inversión de Cartera ",
    "Ingreso Primario - Débito - Funcional, Otra Inversión",
    "Ingreso Secundario",
    "Ingreso Secundario - Crédito",
    "Ingreso Secundario - Débito",
    "Cuenta Capital",
    "Cuenta Capital - Crédito",
    "Cuenta Capital - Débito",
    "Saldo de las cuentas corriente y de capital",
    "Cuenta Financiera",
    "Inversión directa",
    "Inversión directa - Adquisición neta de activos financieros",
    "Inversión directa - Activos - Institucional, Sector Público",
    "Inversión directa - Activos - Institucional, Sector Privado",
    "Inversión directa - Activos - Inst, Priv - Soc. Compraventa",
    "Inversión directa - Activos - Instrumento, Part Capital",
    "Inversión directa - Activos - Instrumento, Deuda",
    "Inversión directa - Pasivos netos incurridos",
    "Inversión directa - Pasivos - Institucional, Sector Público",
    "Inversión directa - Pasivos - Institucional, Sector Privado",
    "Inversión directa - Pasivos - Inst, Priv - Soc. Compraventa",
    "Inversión directa - Pasivos - Instrumento, Part Capital",
    "Inversión directa - Pasivos - Instrumento, Deuda",
    "Inversión de cartera",
    "Inversión de cartera - Adquisición neta de activos fin",
    "Inversión de cartera - Activos - Inst, Sector Público",
    "Inversión de cartera - Activos - Inst, Sector Privado",
    "Inversión de cartera - Activos - Inst, Privado - Soc Comprav",
    "Inversión de cartera - Activos - Instrumento, Part Capital",
    "Inversión de cartera - Activos - Instrumento, Deuda",
    "Inversión de cartera - Pasivos netos incurridos",
    "Inversión de cartera - Pasivos - Inst, Sector Público",
    "Inversión de cartera - Pasivos - Inst, Sector Privado",
    "Inversión de cartera - Pasivos - Inst Priv - Soc Compraventa",
    "Inversión de cartera - Pasivos - Instrumento, Part Capital",
    "Inversión de cartera - Pasivos - Instrumento, Deuda",
    "Derivados financieros - distintos de reservas",
    "Derivados financieros - Sector Público neto",
    "Derivados financieros - Sector Privado neto",
    "Derivados financieros - Privado, Soc. Compraventa",
    "Otra inversión",
    "Otra inversión - Adquisición neta de activos financieros",
    "Otra inversión - Activos - Institucional, Sector Público",
    "Otra inversión - Activos - Institucional, Sector Privado",
    "Otra inversión - Activos - Inst, Priv - Soc. Compraventa",
    "Otra inversión - Activos - Instrumento, Moneda y Depósitos",
    "Otra inversión - Activos - Instrumento, M&D - Banco Central",
    "Otra inversión - Activos - Instrumento, M&D - Bancos",
    "Otra inversión - Activos - Instrumento, M&D - Gob Gral",
    "Otra inversión - Activos - Instrumento, M&D - Otros Sectores",
    "Otra inversión - Activos - Instrumento, Préstamos",
    "Otra inversión - Activos - Instrumento, Préstamos - BC",
    "Otra inversión - Activos - Instrumento, Préstamos - Bancos",
    "Otra inversión - Activos - Instrumento, Préstamos - Gob Gral",
    "Otra inversión - Activos - Instrumento, Préstamos - Otros",
    "Otra inversión - Activos - Instrumento, Créditos y anticipos",
    "Otra inversión - Activos - Instrumento, Créditos - Gob Gral",
    "Otra inversión - Activos - Instrumento, Créditos - Otros",
    "Otra inversión - Activos - Instrumento, Otras cuentas",
    "Otra inversión - Activos - Instrumento, Otras - BC",
    "Otra inversión - Activos - Instrumento, Otras - Bancos",
    "Otra inversión - Activos - Instrumento, Otras - Gob Gral",
    "Otra inversión - Activos - Instrumento, Otras - Otros",
    "Otra inversión - Pasivos netos incurridos",
    "Otra inversión - Pasivos - Institucional, Sector Público",
    "Otra inversión - Pasivos - Institucional, Sector Privado",
    "Otra inversión - Pasivos - Inst, Priv - Soc. Compraventa",
    "Otra inversión - Pasivos - Instrumento, Moneda y Depósitos",
    "Otra inversión - Pasivos - Instrumento, M&D - Banco Central",
    "Otra inversión - Pasivos - Instrumento, M&D - Bancos",
    "Otra inversión - Pasivos - Instrumento, M&D - Gob Gral",
    "Otra inversión - Pasivos - Instrumento, M&D - Otros Sectores",
    "Otra inversión - Pasivos - Instrumento, Préstamos",
    "Otra inversión - Pasivos - Instrumento, Préstamos - BC",
    "Otra inversión - Pasivos - Instrumento, Préstamos - Bancos",
    "Otra inversión - Pasivos - Instrumento, Préstamos - Gob Gral",
    "Otra inversión - Pasivos - Instrumento, Préstamos - Otros",
    "Otra inversión - Pasivos - Instrumento, Créditos y anticipos",
    "Otra inversión - Pasivos - Instrumento, Créditos - Gob Gral",
    "Otra inversión - Pasivos - Instrumento, Créditos - Otros",
    "Otra inversión - Pasivos - Instrumento, Otras cuentas",
    "Otra inversión - Pasivos - Instrumento, Otras - BC",
    "Otra inversión - Pasivos - Instrumento, Otras - Bancos",
    "Otra inversión - Pasivos - Instrumento, Otras - Gob Gral",
    "Otra inversión - Pasivos - Instrumento, Otras - Otros",
    "Derechos Especiales de Giro",
    "Activos de Reserva BCU",
    "Errores y Omisiones",
]



from urllib.error import URLError

import pandas as pd
from pandas.tseries.offsets import MonthEnd

from econuy.utils.operations import (
    get_name_from_function,
    get_download_sources,
    get_names_and_ids,
    get_base_metadata,
)
from econuy.base import Dataset, DatasetMetadata
from econuy import load_dataset
from econuy.utils.retrieval import get_with_ssl_context


def cpi() -> Dataset:
    """Get CPI data.

    Returns
    -------
    Monthly CPI : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = pd.read_excel(sources["main"], usecols="C").dropna(axis=0, how="any")
    output = raw.set_index(
        pd.date_range(start="1937-07-31", freq="ME", periods=len(raw))
    ).rename_axis(None)
    output = output.apply(pd.to_numeric, errors="coerce")

    spanish_names = ["Índice de precios al consumo"]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": "UYU",
        "inflation_adjustment": None,
        "unit": "2022-10=100",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def cpi_core() -> Dataset:
    """Get core CPI data.

    Returns
    -------
    Monthly CPI : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = pd.read_excel(sources["main"], usecols="D")
    output = raw.set_index(
        pd.date_range(start="2022-10-31", freq="ME", periods=len(raw))
    ).rename_axis(None)
    output = output.apply(pd.to_numeric, errors="coerce")

    ids, spanish_names = get_names_and_ids(name, "es")
    output.columns = ids

    base_metadata = get_base_metadata(name)
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def indexed_unit() -> Dataset:
    """Get indexed unit data.

    Returns
    -------
    Monthly indexed unit : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = pd.read_excel(sources["main"], usecols="A:B", skiprows=5, index_col=0)
    output = raw.copy().rename_axis(None).dropna(how="any")
    output.index = pd.to_datetime(output.index, format="%m/%d/%Y")
    output = output.apply(pd.to_numeric, errors="coerce")

    ids, spanish_names = get_names_and_ids(name, "es")
    output.columns = ids

    base_metadata = get_base_metadata(name)
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def cpi_divisions() -> Dataset:
    """Get CPI data by division.

    Returns
    -------
    Monthly CPI by division : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)
    raw = (
        pd.read_excel(sources["main"], usecols="A:D")
        .dropna(axis=0, how="any")
        .assign(
            date=lambda x: x["Año"].astype(str)
            + x["Mes"].astype(str).str.pad(2, "left", "0")
        )
        .pivot(columns="División", values="Indice Total País", index="date")
    )
    output = raw.set_index(
        pd.date_range(start="2010-12-31", freq="ME", periods=len(raw))
    ).rename_axis(None)
    output = output.apply(pd.to_numeric, errors="coerce")

    spanish_names = [
        "Alimentos y bebidas no alcohólicas",
        "Bebidas alcohólicas, tabaco y narcóticos",
        "Ropa y calzado",
        "Vivienda, agua, electricidad, gas y otros combustibles",
        "Mobiliario, enseres domésticos y demás artículos regulares de los hogares",
        "Salud",
        "Transporte",
        "Información y comunicación",
        "Recreación, deporte y cultura",
        "Servicios de educación",
        "Restaurantes y servicios de alojamiento",
        "Seguros y servicios financieros",
        "Cuidado personal, protección social y bienes diversos",
    ]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": "UYU",
        "inflation_adjustment": None,
        "unit": "2022-10=100",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


# @retry(
#     retry_on_exceptions=(HTTPError, URLError),
#     max_calls_total=4,
#     retry_window_after_first_call_in_seconds=60,
# )
# def cpi_classes() -> pd.DataFrame:
#     """Get CPI data by division, group and class.

#     Returns
#     -------
#     Monthly CPI by division, group and class : pd.DataFrame

#     """
#     name = "cpi_classes"

#     raw = []
#     nmonths = 0
#     with pd.ExcelFile(urls[name]["dl"]["main"]) as excel:
#         for sheet in excel.sheet_names:
#             data = (
#                 pd.read_excel(excel, sheet_name=sheet, skiprows=8, nrows=147)
#                 .dropna(how="all")
#                 .dropna(how="all", axis=1)
#             )
#             data["Unnamed: 1"] = np.where(
#                 data["Unnamed: 0"].str.len() == 2,
#                 "Div_" + data["Unnamed: 1"],
#                 np.where(
#                     data["Unnamed: 0"].str.len() == 3,
#                     "Gru_" + data["Unnamed: 1"],
#                     "Cls_" + data["Unnamed: 1"],
#                 ),
#             )
#             data = data.iloc[:, 1:].T
#             data.columns = data.iloc[0]
#             data = data.iloc[1:]
#             data = data.loc[data.index.str.contains("Índice")].reset_index(drop=True).iloc[:, 1:]
#             data.rename(columns={"Cls_Índice General": "Índice General"}, inplace=True)
#             data.index = range(nmonths, nmonths + len(data))
#             data.rename(
#                 columns={
#                     "Div_Bebidas Alcoholicas, Tabaco y Estupefacientes": "Div_Bebidas Alcohólicas, Tabaco y Estupefacientes",
#                     "Div_Muebles, Artículos Para el Hogar y Para la Coservación Oridnaria del Hogar": "Div_Muebles, Artículos Para el Hogar y Para la Conservación Ordinaria del Hogar",
#                     "Gru_Enseñanza no atribuíble a ningún nivel": "Gru_Enseñanza no atribuible a ningún nivel",
#                     "Cls_Enseñanza no atribuíble a ningún nivel": "Cls_Enseñanza no atribuible a ningún nivel",
#                     "Cls_Otros aparatos, articulos y productos para la atención personal": "Cls_Otros aparatos, artículos y productos para la atención personal",
#                 },
#                 inplace=True,
#             )
#             raw.append(data)
#             nmonths = nmonths + len(data)
#     output = pd.concat(raw)
#     output.index = pd.date_range("2011-01-31", freq="ME", periods=len(output))
#     output.rename_axis(None, inplace=True)
#     output = output.apply(pd.to_numeric, errors="coerce")

#     metadata._set(
#         output,
#         area="Precios",
#         currency="-",
#         inf_adj="No",
#         unit="2010-12=100",
#         seas_adj="NSA",
#         ts_type="-",
#         cumperiods=1,
#     )

#     return output


def inflation_expectations() -> Dataset:
    """Get data for the BCU inflation expectations survey.

    Returns
    -------
    Monthly inflation expectations : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    try:
        raw = pd.read_excel(sources["main"], skiprows=9)
    except URLError as err:
        if "SSL: CERTIFICATE_VERIFY_FAILED" in str(err):
            r_bytes = get_with_ssl_context("bcu", sources["main"])
            raw = pd.read_excel(r_bytes, skiprows=9)
    raw = raw.dropna(how="all", axis=1).dropna(thresh=4)
    mask = raw.iloc[-12:].isna().all()
    output = raw.loc[:, ~mask]

    output.columns = (
        ["Fecha"]
        + ["Inflación mensual del mes corriente"] * 5
        + ["Inflación para los próximos 6 meses"] * 5
        + ["Inflación anual del año calendario corriente"] * 5
        + ["Inflación anual de los próximos 12 meses"] * 5
        + ["Inflación anual del año calendario siguiente"] * 5
        + ["Inflación anual de los próximos 24 meses"] * 5
        + ["Inflación anual a dos años calendario"] * 5
    )
    output = output.set_index("Fecha", drop=True)
    output.columns = output.columns + " - " + output.iloc[0]
    output = output.iloc[1:]
    output = output.rename_axis(None)
    output.index = pd.date_range(start="2004-01-31", freq="ME", periods=len(output))
    output = output.apply(pd.to_numeric, errors="coerce")

    spanish_names = output.columns
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": "UYU",
        "inflation_adjustment": None,
        "unit": "%",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": "Flow",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def inflation_expectations_corporate() -> Dataset:
    """Get data for the INE corporate inflation expectations survey.

    Returns
    -------
    Monthly corporate inflation expectations : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = pd.read_excel(sources["main"], skiprows=3, usecols="A:G", index_col=0)
    output = raw.copy()
    output.index = pd.date_range(start="2020-10-31", freq="ME", periods=len(output))
    output = output.apply(pd.to_numeric, errors="coerce")

    ids, spanish_names = get_names_and_ids(name, "es")
    output.columns = ids

    base_metadata = get_base_metadata(name)
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


# @retry(
#     retry_on_exceptions=(HTTPError, URLError),
#     max_calls_total=4,
#     retry_window_after_first_call_in_seconds=60,
# )
# def utilities() -> pd.DataFrame:
#     """Get prices for government-owned utilities.

#     Returns
#     -------
#     Monthly utilities prices : pd.DataFrame

#     """
#     name = "utilities"

#     cpi_1997 = pd.read_excel(urls[name]["dl"]["1997"], skiprows=5, nrows=529).dropna()
#     products = [
#         "    Electricidad",
#         "        Supergas",
#         "        Agua corriente",
#         "      Combustibles Liquidos",
#         "      Servicio Telefonico",
#     ]
#     cpi_1997 = cpi_1997.loc[
#         cpi_1997["Rubros, Agrupaciones, Subrubros, Familias y Artículos"].isin(products)
#     ]
#     cpi_1997 = cpi_1997.T.iloc[1:]
#     cpi_1997.columns = [
#         "Electricidad",
#         "Supergás",
#         "Combustibles líquidos",
#         "Agua corriente",
#         "Servicio telefónico",
#     ]
#     cpi_1997.index = pd.date_range(start="1997-03-31", freq="ME", periods=len(cpi_1997))
#     cpi_1997.rename_axis(None, inplace=True)

#     with pd.ExcelFile(urls[name]["dl"]["2019"]) as excel:
#         cpi_2010 = []
#         for sheet in excel.sheet_names:
#             data = pd.read_excel(excel, sheet_name=sheet, skiprows=8, nrows=640).dropna(how="all")
#             products = [
#                 "04510010",
#                 "04520020",
#                 "07221",
#                 "04410010",
#                 "08300010",
#                 "08300020",
#                 "08300030",
#             ]
#             data = data.loc[
#                 data["Unnamed: 0"].isin(products),
#                 data.columns.str.contains("Indice|Índice", regex=True),
#             ].T
#             data.columns = [
#                 "Agua corriente",
#                 "Electricidad",
#                 "Supergás",
#                 "Combustibles líquidos",
#                 "Servicio de telefonía fija",
#                 "Servicio de telefonía móvil",
#                 "Servicio de Internet",
#             ]
#             cpi_2010.append(data)
#         cpi_2010 = pd.concat(cpi_2010)
#         cpi_2010.index = pd.date_range(start="2010-12-31", freq="ME", periods=len(cpi_2010))

#     output = pd.concat([cpi_1997 / cpi_1997.iloc[-1] * 100, cpi_2010])
#     output = output.loc[~output.index.duplicated(keep="last")]
#     output.at[pd.Timestamp(2010, 12, 31), "Servicio telefónico"] = 100

#     metadata._set(
#         output,
#         area="Precios",
#         currency="-",
#         inf_adj="No",
#         unit="2010-12=100",
#         seas_adj="NSA",
#         ts_type="-",
#         cumperiods=1,
#     )

#     return output


def ppi() -> Dataset:
    """Get PPI data.

    Returns
    -------
    Monthly PPI : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = (
        pd.read_excel(sources["main"], skiprows=7, usecols="H,I,J,K")
        .dropna()
        .rename_axis(None)
    )
    raw.index = pd.date_range(start="1988-01-31", freq="ME", periods=len(raw))
    output = raw.apply(pd.to_numeric, errors="coerce")

    spanish_names = [
        "IPPN Plaza",
        "Producción agropecuaria, forestación y pesca",
        "Explotación de minas y canteras",
        "Industria manufacturera",
    ]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": "UYU",
        "inflation_adjustment": None,
        "unit": "2024-10=100",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": None,
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def nxr_monthly(*args, **kwargs) -> Dataset:
    """Get monthly nominal exchange rate data.

    Returns
    -------
    Monthly nominal exchange rates : Dataset
        Sell rate, monthly average and end of period.

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    daily_data = load_dataset("nxr_daily", *args, **kwargs).data

    output = pd.DataFrame(
        {
            "Tipo de cambio venta, fin de período": daily_data.iloc[:, 0]
            .resample("ME")
            .last(),
            "Tipo de cambio venta, promedio": daily_data.iloc[:, 0]
            .resample("ME")
            .mean(),
        }
    )

    historical = pd.read_excel(
        sources["historical"], skiprows=4, index_col=0, usecols="A,C,F"
    ).dropna(how="any", axis=0)
    historical.columns = [
        "Tipo de cambio venta, fin de período",
        "Tipo de cambio venta, promedio",
    ]
    historical.index = pd.to_datetime(historical.index) + MonthEnd(1)
    historical = historical.apply(pd.to_numeric, errors="coerce")
    historical = historical.loc[:"2001-09-30", :]

    output = pd.concat([historical, output]).rename_axis(None)

    spanish_names = output.columns
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": None,
        "inflation_adjustment": None,
        "unit": "UYU/USD",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": None,
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def nxr_daily() -> Dataset:
    """Get daily nominal exchange rate data.

    Returns
    -------
    Daily nominal exchange rates : Dataset
        Sell rate.

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    output = pd.read_excel(sources["main"], usecols="A,D", index_col=0)
    output.index = pd.to_datetime(output.index, format="%d-%m-%Y")
    output = output.loc[~output.index.duplicated(keep="last")]

    spanish_names = ["Tipo de cambio venta"]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Prices",
        "currency": None,
        "inflation_adjustment": None,
        "unit": "UYU/USD",
        "seasonal_adjustment": None,
        "frequency": "D",
        "time_series_type": None,
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


# @retry(
#     retry_on_exceptions=(HTTPError, URLError),
#     max_calls_total=4,
#     retry_window_after_first_call_in_seconds=60,
# )
# def cpi_measures(pipeline: Optional[Pipeline] = None) -> pd.DataFrame:
#     """
#     Get core CPI, Winsorized CPI, tradabe CPI, non-tradable CPI and residual
#     CPI.

#     Parameters
#     ----------
#     pipeline : econuy.core.Pipeline or None, default None
#         An instance of the econuy Pipeline class.

#     Returns
#     -------
#     Monthly CPI measures : pd.DataFrame

#     """
#     if pipeline is None:
#         pipeline = Pipeline()

#     name = "cpi_measures"

#     xls_10 = pd.ExcelFile(urls[name]["dl"]["2010-"])
#     prod_97 = (
#         pd.read_excel(urls[name]["dl"]["1997"], skiprows=5)
#         .dropna(how="any")
#         .set_index("Rubros, Agrupaciones, Subrubros, Familias y Artículos")
#         .T
#     )

#     weights_97 = pd.read_excel(urls[name]["dl"]["1997_weights"], index_col=0).drop_duplicates(
#         subset="Descripción", keep="first"
#     )
#     weights = pd.read_excel(xls_10, usecols="A:C", skiprows=13, index_col=0).dropna(how="any")
#     weights.columns = ["Item", "Weight"]
#     weights_8 = weights.loc[weights.index.str.len() == 8]

#     sheets = []
#     for sheet in xls_10.sheet_names:
#         raw = pd.read_excel(xls_10, sheet_name=sheet, usecols="D:IN", skiprows=8).dropna(how="all")
#         proc = raw.loc[:, raw.columns.str.contains("Indice|Índice")].dropna(how="all")
#         sheets.append(proc.T)
#     complete_10 = pd.concat(sheets)
#     complete_10 = complete_10.iloc[:, 1:]
#     complete_10.columns = [weights["Item"], weights.index]
#     complete_10.index = pd.date_range(start="2010-12-31", periods=len(complete_10), freq="ME")
#     diff_8 = complete_10.loc[
#         :, complete_10.columns.get_level_values(level=1).str.len() == 8
#     ].pct_change()
#     win = pd.DataFrame(winsorize(diff_8, limits=(0.05, 0.05), axis=1))
#     win.index = diff_8.index
#     win.columns = diff_8.columns.get_level_values(level=1)
#     cpi_win = win.mul(weights_8.loc[:, "Weight"].T)
#     cpi_win = cpi_win.sum(axis=1).add(1).cumprod().mul(100)

#     weights_97["Weight"] = (
#         weights_97["Rubro"]
#         .fillna(weights_97["Agrupación, subrubro, familia"])
#         .fillna(weights_97["Artículo"])
#         .drop(columns=["Rubro", "Agrupación, subrubro, familia", "Artículo"])
#     )
#     prod_97 = prod_97.loc[:, list(cpi_details["1997_base"].keys())]
#     prod_97.index = pd.date_range(start="1997-03-31", periods=len(prod_97), freq="ME")
#     weights_97 = (
#         weights_97[weights_97["Descripción"].isin(cpi_details["1997_weights"])]
#         .set_index("Descripción")
#         .drop(columns=["Rubro", "Agrupación, subrubro, " "familia", "Artículo"])
#     ).div(100)
#     weights_97.index = prod_97.columns
#     prod_10 = complete_10.loc[:, list(cpi_details["2010_base"].keys())]
#     prod_10 = prod_10.loc[:, ~prod_10.columns.get_level_values(level=0).duplicated()]
#     prod_10.columns = prod_10.columns.get_level_values(level=0)
#     weights_10 = (
#         weights.loc[weights["Item"].isin(list(cpi_details["2010_base"].keys()))].drop_duplicates(
#             subset="Item", keep="first"
#         )
#     ).set_index("Item")
#     items = []
#     weights = []
#     for item, weight, details in zip(
#         [prod_10, prod_97], [weights_10, weights_97], ["2010_base", "1997_base"]
#     ):
#         for tradable in [True, False]:
#             items.append(
#                 item.loc[
#                     :, [k for k, v in cpi_details[details].items() if v["Tradable"] is tradable]
#                 ]
#             )
#             aux = weight.loc[
#                 [k for k, v in cpi_details[details].items() if v["Tradable"] is tradable]
#             ]
#             weights.append(aux.div(aux.sum()))
#         for core in [True, False]:
#             items.append(
#                 item.loc[:, [k for k, v in cpi_details[details].items() if v["Core"] is core]]
#             )
#             aux = weight.loc[[k for k, v in cpi_details[details].items() if v["Core"] is core]]
#             weights.append(aux.div(aux.sum()))

#     intermediate = []
#     for item, weight in zip(items, weights):
#         intermediate.append(item.mul(weight.squeeze()).sum(1))

#     output = []
#     for x, y in zip(intermediate[:4], intermediate[4:]):
#         aux = pd.concat(
#             [
#                 y.pct_change().loc[y.index < "2011-01-01"],
#                 x.pct_change().loc[x.index > "2011-01-01"],
#             ]
#         )
#         output.append(aux.fillna(0).add(1).cumprod().mul(100))

#     pipeline.get("cpi")
#     cpi_re = pipeline.dataset
#     cpi_re = cpi_re.loc[cpi_re.index >= "1997-03-31"]
#     output = pd.concat([cpi_re] + output + [cpi_win], axis=1)
#     output.columns = [
#         "Índice de precios al consumo: total",
#         "Índice de precios al consumo: transables",
#         "Índice de precios al consumo: no transables",
#         "Índice de precios al consumo: subyacente",
#         "Índice de precios al consumo: residual",
#         "Índice de precios al consumo: Winsorized 0.05",
#     ]
#     output = output.apply(pd.to_numeric, errors="coerce")
#     metadata._set(
#         output,
#         area="Precios",
#         currency="-",
#         inf_adj="No",
#         unit="2010-12=100",
#         seas_adj="NSA",
#         ts_type="-",
#         cumperiods=1,
#     )
#     output = transform.rebase(output, start_date="2010-12-01", end_date="2010-12-31")
#     output.rename_axis(None, inplace=True)

#     return output



import inspect
import json
import os
from pathlib import Path
from typing import Optional, Dict, List, Tuple

import pandas as pd

from econuy.utils import get_project_root
from econuy.base import Dataset, DatasetMetadata


class DatasetRegistry:
    def __init__(self):
        """
        Initialize the DatasetRegistry by loading the dataset information from a JSON file.
        """
        with open(
            get_project_root() / "retrieval" / "datasets.json", "r", encoding="utf-8"
        ) as f:
            self.registry = json.load(f)

    def __getitem__(self, name: str) -> Dict:
        """
        Retrieve a dataset by its name.

        Parameters
        ----------
        name : str
            The name of the dataset to retrieve.

        Returns
        -------
        dict
            The dataset information.
        """
        return self.registry[name]

    def get_multiple(self, names: List[str]) -> Dict:
        """
        Retrieve multiple datasets by their names.

        Parameters
        ----------
        names : List[str]
            A list of dataset names to retrieve.

        Returns
        -------
        dict
            A dictionary containing the requested datasets.
        """
        return {k: v for k, v in self.registry.items() if k in names}

    def get_available(self) -> Dict:
        """
        Retrieve all available datasets that are not disabled.

        Returns
        -------
        dict
            A dictionary containing all available datasets.
        """
        return {
            k: v
            for k, v in self.registry.items()
            if not v["disabled"] and not v["auxiliary"]
        }

    def get_custom(self) -> Dict:
        """
        Retrieve all custom datasets.

        Returns
        -------
        dict
            A dictionary containing all custom datasets.
        """
        return {k: v for k, v in self.registry.items() if v["custom"]}

    def get_by_area(
        self, area: str, keep_disabled: bool = False, keep_auxiliary: bool = False
    ) -> Dict:
        """
        Retrieve datasets by a specific area, with options to include disabled and auxiliary datasets.

        Parameters
        ----------
        area : str
            The area to filter datasets by.
        keep_disabled : bool, optional
            Whether to include disabled datasets (default is False).
        keep_auxiliary : bool, optional
            Whether to include auxiliary datasets (default is False).

        Returns
        -------
        dict
            A dictionary containing the datasets that match the specified area and options.
        """
        return {
            k: v
            for k, v in self.registry.items()
            if v["area"] == area
            and (keep_disabled or not v["disabled"])
            and (keep_auxiliary or not v["auxiliary"])
        }

    def list_available(self) -> List[str]:
        """
        List the names of all available datasets.

        Returns
        -------
        List[str]
            A list of names of all available datasets.
        """
        return list(self.get_available().keys())

    def list_custom(self) -> List[str]:
        """
        List the names of all custom datasets.

        Returns
        -------
        List[str]
            A list of names of all custom datasets.
        """
        return list(self.get_custom().keys())

    def list_by_area(
        self, area: str, keep_disabled: bool = False, keep_auxiliary: bool = False
    ) -> List[str]:
        """
        List the names of datasets by a specific area, with options to include disabled and auxiliary datasets.

        Parameters
        ----------
        area : str
            The area to filter datasets by.
        keep_disabled : bool, optional
            Whether to include disabled datasets (default is False).
        keep_auxiliary : bool, optional
            Whether to include auxiliary datasets (default is False).

        Returns
        -------
        List[str]
            A list of names of datasets that match the specified area and options.
        """
        return list(self.get_by_area(area, keep_disabled, keep_auxiliary).keys())


REGISTRY = DatasetRegistry()


def get_name_from_function() -> str:
    return inspect.currentframe().f_back.f_code.co_name


def get_download_sources(name: str) -> Dict:
    return REGISTRY[name]["sources"]["downloads"]


def get_base_metadata(name: str) -> Dict:
    return REGISTRY[name]["base_metadata"]


def get_names_and_ids(name: str, language: str = "es") -> Tuple[List[str], List[Dict]]:
    ids_names = REGISTRY[name]["indicator_ids"]
    ids_names = {k: v[language] for k, v in ids_names.items()}
    language_names = [{"es": x} for x in ids_names.values()]
    ids = [f"{name}_{i}" for i in ids_names.keys()]
    return ids, language_names


def get_data_dir() -> Path:
    data_dir = os.getenv("ECONUY_DATA_DIR", "") or Path.home() / ".cache" / "econuy"
    data_dir = Path(data_dir)
    os.environ["ECONUY_DATA_DIR"] = data_dir.as_posix()
    data_dir.mkdir(parents=True, exist_ok=True, mode=0o755)
    return data_dir


def read_dataset(name: str, data_dir: Path) -> Optional[Dataset]:  # noqa: F821
    dataset_path = (data_dir / name).with_suffix(".csv")
    metadata_path = (data_dir / f"{name}_metadata").with_suffix(".json")
    if not dataset_path.exists() or not metadata_path.exists():
        return None

    dataset = pd.read_csv(dataset_path, index_col=0, parse_dates=True)
    metadata = DatasetMetadata.from_json(metadata_path)
    dataset = Dataset(name, dataset, metadata)
    return dataset



from typing import Union, Tuple
from datetime import datetime

import pandas as pd


def _rebase(
    data: pd.DataFrame,
    metadata: "Metadata",  # type: ignore # noqa: F821
    start_date: Union[str, datetime],
    end_date: Union[str, datetime, None] = None,
    base: float = 100.0,
) -> Tuple[pd.DataFrame, "Metadata"]:  # type: ignore # noqa: F821
    metadata = metadata.copy()
    if end_date is None:
        m_end = None
        start_date = data.iloc[
            data.index.get_indexer([start_date], method="nearest")
        ].index[0]
        output = data.apply(lambda x: x / x.loc[start_date] * base)
        if isinstance(start_date, str):
            start_date = datetime.strptime(start_date, "%Y-%m-%d")
        if not isinstance(base, int):
            if base.is_integer():
                base = int(base)
        m_start = start_date.strftime("%Y-%m")
        metadata.update_dataset_metadata({"unit": f"{m_start}={base}"})

    else:
        output = data.apply(lambda x: x / x[start_date:end_date].mean() * base)
        if isinstance(start_date, str):
            start_date = datetime.strptime(start_date, "%Y-%m-%d")
        if isinstance(end_date, str):
            end_date = datetime.strptime(end_date, "%Y-%m-%d")
        m_start = start_date.strftime("%Y-%m")
        m_end = end_date.strftime("%Y-%m")
        if not isinstance(base, int):
            if base.is_integer():
                base = int(base)
        if m_start == m_end:
            metadata.update_dataset_metadata({"unit": f"{m_start}={base}"})
        else:
            metadata.update_dataset_metadata({"unit": f"{m_start}_{m_end}={base}"})

    metadata.add_transformation_step(
        {"rebase": {"start_date": m_start, "end_date": m_end, "base": base}}
    )

    return output, metadata



import re
import os
import tempfile
import time
import zipfile
import datetime as dt
import ssl
from io import BytesIO
from os import path, listdir

import pandas as pd
import numpy as np
import httpx
from pandas.tseries.offsets import MonthEnd
from dotenv import load_dotenv

from econuy import load_dataset
from econuy.base import Dataset, DatasetMetadata
from econuy.utils.chromedriver import _build
from econuy.utils.operations import get_download_sources, get_name_from_function
from econuy.utils.retrieval import get_certs_path


load_dotenv()
FRED_API_KEY = os.environ.get("FRED_API_KEY")


def regional_gdp() -> Dataset:
    """Get seasonally adjusted real GDP for Argentina and Brazil.

    Returns
    -------
    Quarterly real GDP : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    driver = _build()
    driver.get(sources["arg_new"])
    time.sleep(5)
    source = driver.page_source
    driver.quit()
    url = re.findall(r"/ftp/cuadros/economia/.+desest.+\.xls", source)[0]
    full_url = f"https://www.indec.gob.ar{url}"
    arg = pd.read_excel(full_url, skiprows=3, usecols="C").dropna(how="all")
    arg.index = pd.date_range(start="2004-03-31", freq="QE-DEC", periods=len(arg))
    arg_old = pd.read_excel(sources["arg_old"], skiprows=7, usecols="D").dropna(
        how="all"
    )
    arg_old.index = pd.date_range(
        start="1993-03-31", freq="QE-DEC", periods=len(arg_old)
    )
    arg = pd.concat([arg, arg_old], axis=1)
    for row in reversed(range(len(arg))):
        if pd.isna(arg.iloc[row, 0]):
            arg.iloc[row, 0] = (
                arg.iloc[row, 1] / arg.iloc[row + 1, 1] * arg.iloc[row + 1, 0]
            )
    arg = arg.iloc[:, [0]]

    r = httpx.get(sources["bra"])
    temp_dir = tempfile.TemporaryDirectory()
    with zipfile.ZipFile(BytesIO(r.content), "r") as f:
        f.extractall(path=temp_dir.name)
    path_temp = path.join(temp_dir.name, listdir(temp_dir.name)[0])
    bra = pd.read_excel(
        path_temp, usecols="Q", skiprows=3, sheet_name="Val encad preços 95 com ajuste"
    )
    bra.index = pd.date_range(start="1996-03-31", freq="QE-DEC", periods=len(bra))

    output = pd.concat([arg, bra], axis=1).div(1000)
    output.columns = ["Argentina", "Brasil"]
    output = output.rename_axis(None)

    spanish_names = output.columns
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Regional",
        "currency": "-",
        "inflation_adjustment": "Const.",
        "unit": "Billions",
        "seasonal_adjustment": "Seasonally adjusted",
        "frequency": "QE-DEC",
        "time_series_type": "Flow",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    for indicator, currency in zip(ids, ["ARS", "BRL"]):
        metadata.update_indicator_metadata_value(indicator, "currency", currency)
    dataset = Dataset(name, output, metadata)

    return dataset


def regional_monthly_gdp() -> Dataset:
    """Get monthly GDP data.

    Countries/aggregates selected are Argentina and Brazil.

    Returns
    -------
    Monthly GDP : pd.DataFrame

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    arg = pd.read_excel(sources["arg"], usecols="E", skiprows=3).dropna(how="all")
    arg.index = pd.date_range(start="2004-01-31", freq="ME", periods=len(arg))

    bra = pd.read_csv(sources["bra"], sep=";", index_col=0, decimal=",")
    bra.index = pd.date_range(start="2003-01-31", freq="ME", periods=len(bra))

    output = pd.concat([arg, bra], axis=1)
    output.columns = ["Argentina", "Brasil"]
    output = output.rename_axis(None)

    spanish_names = output.columns
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids
    spanish_names = [{"es": x} for x in spanish_names]

    base_metadata = {
        "area": "Regional",
        "currency": "-",
        "inflation_adjustment": "Const.",
        "unit": "-",
        "seasonal_adjustment": "Seasonally adjusted",
        "frequency": "ME",
        "time_series_type": "Flow",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    for indicator, currency in zip(ids, ["ARS", "BRL"]):
        metadata.update_indicator_metadata_value(indicator, "currency", currency)
    dataset = Dataset(name, output, metadata).rebase("2010-01-01", "2010-12-31")
    dataset.metadata.update_dataset_metadata({"unit": "2010=100"})
    dataset.transformed = False

    return dataset


def regional_cpi() -> pd.DataFrame:
    """Get consumer price index for Argentina and Brazil.

    Returns
    -------
    Monthly CPI : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    certs = get_certs_path("bcra")
    ssl_context = ssl.create_default_context(cafile=str(certs))
    arg = httpx.get(
        sources["ar"].format(
            end_date=dt.datetime.now().strftime("%Y-%m-%d"),
        ),
        verify=ssl_context,
    )
    arg = pd.read_html(arg.content)[0]
    arg.set_index("Fecha", drop=True, inplace=True)
    arg.index = pd.to_datetime(arg.index, format="%d/%m/%Y")
    arg.columns = ["nivel"]
    arg = arg.divide(10)

    arg_unoff = pd.read_excel(sources["ar_unofficial"])
    arg_unoff.set_index("date", drop=True, inplace=True)
    arg_unoff.index = arg_unoff.index + MonthEnd(0)
    arg_unoff = arg_unoff.loc[
        (arg_unoff.index >= "2006-12-01") & (arg_unoff.index <= "2016-12-01"), "index"
    ]
    arg_unoff = arg_unoff.to_frame().pct_change(periods=1).multiply(100).dropna()
    arg_unoff.columns = ["nivel"]
    arg = (
        pd.concat([arg, arg_unoff])
        .reset_index()
        .drop_duplicates(subset="index", keep="last")
        .set_index("index", drop=True)
        .sort_index()
    )
    arg = arg.divide(100).add(1).cumprod()

    bra_r = httpx.get(sources["bra"].format(date=dt.datetime.now().strftime("%Y%m")))
    bra = pd.DataFrame(bra_r.json())[["v"]]
    bra.index = pd.date_range(start="1979-12-31", freq="ME", periods=len(bra))
    bra = bra.apply(pd.to_numeric, errors="coerce")
    bra = bra.divide(100).add(1).cumprod()

    output = pd.concat([arg, bra], axis=1)
    output.columns = ["Argentina", "Brasil"]
    output = output.rename_axis(None)

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "-",
        "inflation_adjustment": None,
        "unit": "-",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": "Flow",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    for indicator, currency in zip(ids, ["ARS", "BRL"]):
        metadata.update_indicator_metadata_value(indicator, "currency", currency)
    dataset = Dataset(name, output, metadata).rebase("2010-01-01", "2010-12-31")
    dataset.metadata.update_dataset_metadata({"unit": "2010=100"})
    dataset.transformed = False

    return dataset


def regional_embi_spreads() -> Dataset:
    """Get EMBI spread for Argentina, Brazil and the EMBI Global.

    Returns
    -------
    Daily 10-year government bond spreads : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    raw = pd.read_excel(sources["main"], usecols="A:B,E,G", skiprows=1, index_col=0)
    output = (
        raw.loc[~pd.isna(raw.index)]
        .mul(100)
        .rename(columns={"Global": "EMBI Global"})[
            ["Argentina", "Brasil", "EMBI Global"]
        ]
    )
    output.index = pd.to_datetime(output.index)
    output = output.apply(pd.to_numeric, errors="coerce").sort_index()
    output = output.rename_axis(None)

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "USD",
        "inflation_adjustment": None,
        "unit": "Bps",
        "seasonal_adjustment": None,
        "frequency": "D",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def regional_embi_yields(*args, **kwargs) -> Dataset:
    """Get EMBI yields for Argentina, Brazil and the EMBI Global.

    Yields are calculated by adding EMBI spreads to the 10-year US Treasury
    bond rate.

    Returns
    -------
    Daily 10-year government bonds interest rates : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    if FRED_API_KEY is None:
        raise ValueError(
            "FRED_API_KEY not found. Get one at https://fredaccount.stlouisfed.org/apikeys and set it as an environment variable."
        )

    r = httpx.get(sources["treasury"].format(FRED_API_KEY))
    treasuries = pd.DataFrame.from_records(r.json()["observations"]).set_index("date")[
        ["value"]
    ]
    treasuries.index = pd.to_datetime(treasuries.index)
    treasuries["value"] = pd.to_numeric(
        treasuries["value"], errors="coerce"
    ).interpolate()

    spreads = load_dataset("regional_embi_spreads", *args, **kwargs).to_named()

    treasuries = treasuries.reindex(spreads.index).interpolate(
        method="linear", limit_direction="forward"
    )
    output = spreads.div(100).add(treasuries.squeeze(), axis=0)
    output = output.rename_axis(None)

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "USD",
        "inflation_adjustment": None,
        "unit": "Rate",
        "seasonal_adjustment": None,
        "frequency": "D",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def regional_nxr() -> Dataset:
    """Get USDARS and USDBRL.

    Returns
    -------
    Daily exchange rates : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    arg = []
    for dollar in ["ar", "ar_unofficial"]:
        r = httpx.get(
            sources[dollar].format(date=dt.datetime.now().strftime("%d-%m-%Y")),
            headers={
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36"
            },
        )
        aux = pd.DataFrame(r.json())[[0, 2]]
        aux = aux.set_index(0, drop=True)
        aux = aux.drop("Fecha")
        aux = aux.replace(",", ".", regex=True).apply(pd.to_numeric)
        aux.index = pd.to_datetime(aux.index, format="%d/%m/%Y")
        aux = aux.sort_index()
        aux.columns = [dollar]
        arg.append(aux)
    arg = arg[0].join(arg[1], how="left")
    arg.columns = ["Argentina - oficial", "Argentina - informal"]

    r = httpx.get(sources["bra"])
    bra = pd.DataFrame(r.json())
    bra = [(x["VALDATA"], x["VALVALOR"]) for x in bra["value"]]
    bra = pd.DataFrame.from_records(bra).dropna(how="any")
    bra = bra.set_index(0)
    bra.index = pd.to_datetime(bra.index.str[:-4]).tz_localize(None)
    bra.columns = ["Brasil"]

    output = arg.join(bra, how="left").interpolate(method="linear", limit_area="inside")
    output = output.rename_axis(None)

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "USD",
        "inflation_adjustment": None,
        "unit": "-",
        "seasonal_adjustment": None,
        "frequency": "ME",
        "time_series_type": "Flow",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    dataset = Dataset(name, output, metadata)

    return dataset


def regional_policy_rates() -> Dataset:
    """Get central bank policy interest rates data.

    Countries/aggregates selected are Argentina and Brazil.

    Returns
    -------
    Daily policy interest rates : Dataset

    """
    name = get_name_from_function()
    sources = get_download_sources(name)

    r = httpx.get(sources["main"])
    temp_dir = tempfile.TemporaryDirectory()
    with zipfile.ZipFile(BytesIO(r.content), "r") as f:
        f.extractall(path=temp_dir.name)
        path_temp = path.join(temp_dir.name, "WS_CBPOL_csv_row.csv")
        raw = pd.read_csv(path_temp, index_col=0)
    output = raw.loc[:, lambda x: x.columns.str.contains("D:Daily")]
    output.columns = output.iloc[0]
    output = output.loc[:, ["AR:Argentina", "BR:Brazil"]].iloc[8:].dropna(how="all")
    output.columns = ["Argentina", "Brasil"]
    output = output.apply(pd.to_numeric, errors="coerce").interpolate(
        method="linear", limit_area="inside"
    )
    output.index = pd.to_datetime(output.index)
    output = output.rename_axis(None)

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "-",
        "inflation_adjustment": None,
        "unit": "Rate",
        "seasonal_adjustment": None,
        "frequency": "D",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    for indicator, currency in zip(ids, ["ARS", "BRL"]):
        metadata.update_indicator_metadata_value(indicator, "currency", currency)
    dataset = Dataset(name, output, metadata)

    return dataset


# def regional_stock_markets() -> pd.DataFrame:
#     """Get stock market index data in USD terms.

#     Indexes selected are MERVAL and BOVESPA.

#     Parameters
#     ----------
#     pipeline : econuy.core.Pipeline or None, default None
#         An instance of the econuy Pipeline class.

#     Returns
#     -------
#     Daily stock market index in USD terms: pd.DataFrame

#     """
#     name = get_name_from_function()
#     sources = get_download_sources(name)

#     yahoo = []
#     for series in ["arg", "bra"]:
#         aux = pd.read_csv(
#             sources[series].format(timestamp=dt.datetime.now().timestamp().__round__()),
#             index_col=0,
#             usecols=[0, 4],
#             parse_dates=True,
#         )
#         aux.columns = [series]
#         yahoo.append(aux)
#     output = pd.concat(yahoo, axis=1).interpolate(method="linear", limit_area="inside")
#     output.columns = ["MERVAL", "BOVESPA"]
#     output.rename_axis(None, inplace=True)
#     metadata._set(
#         output,
#         area="Global",
#         currency="USD",
#         inf_adj="No",
#         seas_adj="NSA",
#         ts_type="-",
#         cumperiods=1,
#     )
#     metadata._modify_multiindex(output, levels=[3], new_arrays=[["ARS", "BRL"]])
#     output = rebase(output, start_date="2019-01-02")

#     return output


def regional_rxr(*args, **kwargs) -> Dataset:
    """Get real exchange rates vis-á-vis the US dollar for Argentina and Brasil .

    Returns
    -------
    Monthly real exchange rate : Dataset

    """
    name = get_name_from_function()
    proc = _ifs(*args, **kwargs)

    output = pd.DataFrame()
    output["Argentina"] = (
        proc["Argentina - oficial"] * proc["US.PCPI_IX"] / proc["ARG CPI"]
    )
    output["Brasil"] = proc["Brasil"] * proc["US.PCPI_IX"] / proc["BRA CPI"]
    output = output.rename_axis(None).dropna(how="all")

    spanish_names = output.columns
    spanish_names = [{"es": x} for x in spanish_names]
    ids = [f"{name}_{i}" for i in range(output.shape[1])]
    output.columns = ids

    base_metadata = {
        "area": "Regional",
        "currency": "USD",
        "inflation_adjustment": None,
        "unit": "-",
        "seasonal_adjustment": None,
        "frequency": "D",
        "time_series_type": "Stock",
        "cumulative_periods": 1,
        "transformations": [],
    }
    metadata = DatasetMetadata.from_cast(
        name, base_metadata, output.columns, spanish_names
    )
    for indicator, unit in zip(ids, ["ARS/USD", "BRL/USD"]):
        metadata.update_indicator_metadata_value(indicator, "unit", unit)
    dataset = Dataset(name, output, metadata).rebase("2019-01-01", "2019-01-31")
    dataset.metadata.update_dataset_metadata({"unit": "2019-01=100"})
    dataset.transformed = False

    return dataset


def _ifs(*args, **kwargs) -> pd.DataFrame:
    """Get extra CPI and exchange rate data from the IMF IFS.

    Returns
    -------
    IMF data : pd.DataFrame
        CPI and XR for the US, Brazil and Argentina.

    """
    url_ = "http://dataservices.imf.org/REST/SDMX_JSON.svc/CompactData/IFS/M"
    url_extra = "?startPeriod=1970&endPeriod="
    ifs = []
    for country in ["US", "BR", "AR"]:
        for indicator in ["PCPI_IX", "ENDA_XDC_USD_RATE"]:
            base_url = (
                f"{url_}.{country}.{indicator}.{url_extra}{dt.datetime.now().year}"
            )
            r_json = httpx.get(base_url, timeout=30).json()
            data = r_json["CompactData"]["DataSet"]["Series"]["Obs"]
            try:
                data = pd.DataFrame(data)
                data.set_index("@TIME_PERIOD", drop=True, inplace=True)
            except ValueError:
                data = pd.DataFrame(
                    np.nan,
                    index=pd.date_range(
                        start="1970-01-01", end=dt.datetime.now(), freq="ME"
                    ),
                    columns=[f"{country}.{indicator}"],
                )
            if "@OBS_STATUS" in data.columns:
                data.drop("@OBS_STATUS", inplace=True, axis=1)
            data.index = pd.to_datetime(data.index, format="%Y-%m") + MonthEnd(1)
            data.columns = [f"{country}.{indicator}"]
            ifs.append(data)
    ifs = pd.concat(ifs, axis=1, sort=True).apply(pd.to_numeric)

    xr = load_dataset("regional_nxr", *args, **kwargs).resample("ME", "mean").to_named()
    prices = load_dataset("regional_cpi", *args, **kwargs).to_named()
    prices.columns = ["ARG CPI", "BRA CPI"]

    proc = pd.concat([xr, prices, ifs], axis=1)
    proc["Argentina - oficial"] = np.where(
        pd.isna(proc["Argentina - oficial"]),
        proc["AR.ENDA_XDC_USD_RATE"],
        proc["Argentina - oficial"],
    )
    proc["Argentina - informal"] = np.where(
        pd.isna(proc["Argentina - informal"]),
        proc["AR.ENDA_XDC_USD_RATE"],
        proc["Argentina - informal"],
    )
    proc["Brasil"] = np.where(
        pd.isna(proc["Brasil"]), proc["BR.ENDA_XDC_USD_RATE"], proc["Brasil"]
    )
    proc = proc[
        [
            "Argentina - oficial",
            "Argentina - informal",
            "Brasil",
            "ARG CPI",
            "BRA CPI",
            "US.PCPI_IX",
        ]
    ]

    return proc



import pandas as pd
import numpy as np

from econuy.utils.exceptions import InvalidTransformation


def error_handler(df: pd.DataFrame, errors: str, msg: str = None) -> pd.DataFrame:
    if errors == "coerce":
        return pd.DataFrame(data=np.nan, index=df.index, columns=df.columns)
    elif errors == "ignore":
        return df
    elif errors == "raise":
        if msg is None:
            msg = ""
        raise InvalidTransformation(msg)



from typing import Tuple

import pandas as pd


def _chg_diff(
    data: pd.DataFrame,
    metadata: "Metadata",  # type: ignore # noqa: F821
    operation: str = "chg",
    period: str = "last",
) -> Tuple[pd.DataFrame, "Metadata"]:  # type: ignore # noqa: F821
    from econuy.transform.rolling import _rolling

    indicators = metadata.indicator_ids
    metadata = metadata.copy()
    # We get the first one because we validated that all indicators have the same metadata, or pass them one by one
    single_metadata = metadata.indicator_metadata[indicators[0]]
    time_series_type = single_metadata["time_series_type"]
    inferred_freq = pd.infer_freq(data.index)

    type_change = {
        "last": {
            "chg": [lambda x: x.pct_change(periods=1), "Pct. change"],
            "diff": [lambda x: x.diff(periods=1), "Change"],
        },
        "inter": {
            "chg": [
                lambda x: x.pct_change(periods=last_year),
                "Pct. change YoY",
            ],
            "diff": [lambda x: x.diff(periods=last_year), "Change YoY"],
        },
        "annual": {
            "chg": [lambda x: x.pct_change(periods=last_year), "Pct. change annual"],
            "diff": [lambda x: x.diff(periods=last_year), "Change annual"],
        },
    }

    if inferred_freq in ["ME"]:
        last_year = 12
    elif inferred_freq in ["QE", "QE-DEC"]:
        last_year = 4
    elif inferred_freq in ["YE", "YE-DEC"]:
        last_year = 1
    else:
        raise ValueError(
            "The dataframe needs to have a frequency of ME "
            "(month end), QQ (quarter end) or YE (year end)"
        )

    if period == "annual":
        if time_series_type == "Stock":
            output = data.apply(type_change[period][operation][0])
        else:
            output, metadata = _rolling(data, metadata, operation="sum")
            output = output.apply(type_change[period][operation][0])
    else:
        output = data.apply(type_change[period][operation][0])

    if operation == "chg":
        output = output.multiply(100)

    metadata.update_dataset_metadata({"unit": type_change[period][operation][1]})
    metadata.add_transformation_step(
        {"chg_diff": {"operation": operation, "period": period}}
    )

    return output, metadata



import ssl
from io import BytesIO
from typing import Literal
from pathlib import Path

import httpx

from econuy.utils.operations import get_project_root


def get_certs_path(source: Literal["bcu", "ine", "inac", "bcra"]) -> Path:
    return Path(get_project_root(), "utils", "files", f"{source}_certs.pem")


def get_with_ssl_context(
    source: Literal["bcu", "ine", "inac", "bcra"], url: str
) -> bytes:
    certs_path = get_certs_path(source)
    ssl_context = ssl.create_default_context(cafile=str(certs_path))

    r = httpx.get(url, verify=ssl_context)
    return BytesIO(r.content)



from typing import Optional, Tuple

import pandas as pd


def _rolling(
    data: pd.DataFrame,
    metadata: "Metadata",  # type: ignore # noqa: F821
    window: Optional[int] = None,
    operation: str = "sum",
) -> Tuple[pd.DataFrame, "Metadata"]:  # type: ignore # noqa: F821
    metadata = metadata.copy()
    # We get the first one because we validated that all indicators have the same metadata, or pass them one by one

    pd_frequencies = {
        "YE": 1,
        "YE-DEC": 1,
        "QE": 4,
        "QE-DEC": 4,
        "ME": 12,
        "W": 52,
        "W-SUN": 52,
        "2W": 26,
        "2W-SUN": 26,
        "B": 260,
        "D": 365,
    }

    window_operation = {
        "sum": lambda x: x.rolling(window=window, min_periods=window).sum(),
        "mean": lambda x: x.rolling(window=window, min_periods=window).mean(),
    }

    if window is None:
        inferred_freq = pd.infer_freq(data.index)
        window = pd_frequencies[inferred_freq]

    output = data.apply(window_operation[operation])
    metadata.update_dataset_metadata({"cumulative_periods": window})
    metadata.add_transformation_step(
        {"rolling": {"window": window, "operation": operation}}
    )

    return output, metadata



import os
from pathlib import Path

import pytest

from econuy.core import Pipeline
from econuy.utils import get_project_root, metadata, operations


def trim_rows(input: str, output: str, rows: int = 10):
    for f in os.listdir(input):
        if f.endswith(".csv") and "commodity_weights" not in f:
            df = operations._read(Path(input, f), file_fmt="csv", multiindex="included")
            df = df.reindex(df.index[:-rows])
            operations._save(df, Path(output, f))


p = Pipeline()
all_datasets = list(p.available_datasets)


@pytest.mark.parametrize(
    "dataset",
    all_datasets,
)
def test_retrieval(dataset):
    """Download every available dataset and compare with a previous version.

    Up to 5% deviations are accepted.
    """
    location = Path(get_project_root().parent, "tests/test-data")
    p = Pipeline(location=location, always_save=False)
    p.get(dataset)
    test_path = Path(location, f"{dataset}.csv")
    test = operations._read(test_path)
    metadata._set(test)
    compare = p.dataset.reindex(test.index)
    div = test / compare
    final = div[((div > 1.05) | (div < 0.95)).any(axis=1)]
    assert len(final) == 0



from os import path

import numpy as np
import pandas as pd
import pytest
from sqlalchemy import create_engine

from econuy.session import Session
from econuy.utils import metadata

CUR_DIR = path.abspath(path.dirname(__file__))
TEST_DIR = path.join(CUR_DIR, "test-data")
TEST_CON = create_engine("sqlite://").connect()


def create_dummy_df(
    freq,
    periods=200,
    area="Test",
    currency="Test",
    inf_adj="Test",
    unit="Test",
    seas_adj="Test",
    ts_type="Test",
    cumperiods=1,
):
    dates = pd.date_range("2000-01-31", periods=periods, freq=freq)
    cols = ["A", "B", "C"]
    data = np.hstack(
        [
            np.random.uniform(-100, 100, [periods, 1]),
            np.random.uniform(1, 50, [periods, 1]),
            np.random.uniform(-100, -50, [periods, 1]),
        ]
    )
    output = pd.DataFrame(index=dates, columns=cols, data=data)
    metadata._set(
        output,
        area=area,
        currency=currency,
        inf_adj=inf_adj,
        unit=unit,
        seas_adj=seas_adj,
        ts_type=ts_type,
        cumperiods=cumperiods,
    )
    return output


@pytest.mark.parametrize(
    "freq,ts_type,operation,period,pd_per,pd_rol",
    [
        ("YE-DEC", "Flujo", "chg", "last", 1, 1),
        ("QE-DEC", "Stock", "diff", "inter", 4, 1),
        ("ME", "Flujo", "chg", "annual", 12, 12),
    ],
)
def test_chg_diff(freq, ts_type, operation, period, pd_per, pd_rol):
    df = create_dummy_df(freq=freq, ts_type=ts_type)
    s = Session(TEST_CON)
    s._datasets.update({"dummy": df})
    s.chg_diff(select="all", operation=operation, period=period)
    compare = s.datasets["dummy"]
    if ts_type == "Flujo" and period == "annual" and pd_rol > 1:
        df = df.rolling(window=pd_rol, min_periods=pd_rol).sum()
    if operation == "chg":
        df = df.pct_change(pd_per).mul(100)
    else:
        df = df.diff(pd_per)
    compare.index, compare.columns = df.index, df.columns
    assert df.equals(compare)


@pytest.mark.parametrize(
    "freq,ts_type,operation,window,pd_rol",
    [("M", "Flujo", "sum", 3, 3), ("QE-DEC", "Flujo", "mean", None, 4)],
)
def test_rolling(freq, ts_type, operation, window, pd_rol):
    df = create_dummy_df(freq=freq, ts_type=ts_type)
    s = Session()
    s._datasets["dummy"] = df
    s.rolling(window=window, operation=operation, select="dummy")
    compare = s.datasets["dummy"]
    if operation == "sum":
        df = df.rolling(window=pd_rol, min_periods=pd_rol).sum()
    else:
        df = df.rolling(window=pd_rol, min_periods=pd_rol).mean()
    compare.index, compare.columns = df.index, df.columns
    assert df.equals(compare)


@pytest.mark.parametrize(
    "freq,ts_type,periods,cumperiods,rule,operation",
    [
        ("ME", "Flujo", 192, 1, "YE-DEC", "sum"),
        ("QE-DEC", "Flujo", 102, 2, "YE-DEC", "mean"),
        ("QE-DEC", "Stock", 102, 1, "YE-DEC", "last"),
        ("YE-DEC", "Flujo", 102, 1, "ME", "upsample"),
    ],
)
def test_resample(freq, ts_type, periods, cumperiods, rule, operation):
    df = create_dummy_df(
        freq=freq, ts_type=ts_type, periods=periods, cumperiods=cumperiods
    )
    s = Session()
    s._datasets["dummy"] = df
    s.resample(rule=rule, operation=operation, select=0)
    compare = s._datasets["dummy"]
    if operation == "sum":
        resampled = df.resample(rule=rule).sum()
    elif operation == "mean":
        resampled = df.resample(rule=rule).mean()
    elif operation == "last":
        resampled = df.resample(rule=rule).last()
    else:
        resampled = df.resample(rule=rule).last().interpolate("linear")

    pd_freqs = {"M": 12, "ME": 12, "QE-DEC": 4, "Q-DEC": 4, "A-DEC": 1, "YE-DEC": 1}
    if periods % pd_freqs[freq] != 0 and operation != "upsample":
        rows = periods // pd_freqs[freq]
        resampled = resampled.iloc[:rows, :]
    compare.index, compare.columns = resampled.index, resampled.columns
    assert resampled.equals(compare)


@pytest.mark.parametrize(
    "component,method,fallback,trading,outlier",
    [
        ("trend", "x13", "loess", True, True),
        ("seas", "x13", "ma", True, False),
        ("trend", "loess", "ma", False, True),
        ("trend", "ma", "loess", False, False),
    ],
)
def test_decompose(component, method, fallback, trading, outlier):
    df = pd.DataFrame(
        index=pd.date_range("2000-01-01", periods=100, freq="QE-DEC"),
        data=np.random.exponential(2, 100).cumsum(),
        columns=["Exponential"],
    )
    df["Real"] = df["Exponential"]
    df.loc[df.index.month == 12, "Real"] = df.loc[
        df.index.month == 12, "Real"
    ].multiply(np.random.uniform(1.06, 1.14))
    df.loc[df.index.month == 6, "Real"] = df.loc[df.index.month == 6, "Real"].multiply(
        np.random.uniform(0.94, 0.96)
    )
    df.loc[df.index.month == 3, "Real"] = df.loc[df.index.month == 3, "Real"].multiply(
        np.random.uniform(1.04, 1.06)
    )
    df.drop("Exponential", axis=1, inplace=True)
    df = df.add(np.random.normal(0, 1, 100), axis=0)
    metadata._set(df, seas_adj="NSA")
    s = Session()
    s._datasets["test"] = df
    s.decompose(
        component=component,
        trading=trading,
        method=method,
        outlier=outlier,
        fallback=fallback,
        ignore_warnings=True,
    )
    assert s._datasets["test"].std().values <= df.std().values


@pytest.mark.parametrize(
    "freq,start_date,end_date,base",
    [("M", "2004-01-01", None, 100), ("QE-DEC", "2004-01-01", "2005-01-01", 100.1)],
)
def test_rebase(freq, start_date, end_date, base):
    df = create_dummy_df(freq=freq)
    s = Session()
    s._datasets["dummy"] = df
    s.rebase(start_date=start_date, end_date=end_date, base=base, select=[0])
    compare = s._datasets["dummy"]
    if end_date is None:
        start_date = pd.to_datetime(start_date)
        start = df.index.to_series().sub(start_date).abs().idxmin()
        df = df / df.loc[start] * base
    else:
        df = df / df.loc[start_date:end_date].mean() * base
    compare.index, compare.columns = df.index, df.columns
    assert df.equals(compare)


@pytest.mark.parametrize(
    "freq,ts_type,periods,cumperiods",
    [
        ("D", "Flujo", 600, 1),
        ("M", "Stock", 48, 1),
        ("W-SUN", "Stock", 200, 1),
        ("QE-DEC", "Flujo", 24, 4),
    ],
)
def test_convert_usd(freq, ts_type, periods, cumperiods):
    df = create_dummy_df(
        freq=freq,
        periods=periods,
        ts_type=ts_type,
        currency="UYU",
        cumperiods=cumperiods,
    )
    s = Session(location=TEST_DIR, download=False)
    s._datasets["dummy"] = df
    s.convert(flavor="usd")
    s.get("nxr_monthly")
    compare = s.datasets["dummy"]
    nxr = s.datasets["nxr_monthly"]
    proc_freq = freq

    if freq in ["D", "W", "B", "W-SUN"]:
        if ts_type == "Stock":
            df = df.resample("ME").last()
        else:
            df = df.resample("ME").sum()
        proc_freq = "M"

    if ts_type == "Stock":
        nxr = nxr.resample(proc_freq).last().iloc[:, [1]]
    else:
        nxr = nxr.resample(proc_freq).mean().iloc[:, [0]]
        nxr = nxr.rolling(window=cumperiods, min_periods=cumperiods).mean()

    nxr = nxr.reindex(df.index).iloc[:, 0]
    df = df.div(nxr, axis=0)
    compare.index, compare.columns = df.index, df.columns
    assert df.equals(compare)


@pytest.mark.parametrize(
    "freq,periods,ts_type,start_date,end_date",
    [
        ("M", 100, "Flujo", None, None),
        ("D", 1200, "Flujo", "2002-01-01", None),
        ("QE-DEC", 40, "Stock", "2002-01-01", "2002-12-31"),
    ],
)
def test_convert_real(freq, periods, ts_type, start_date, end_date):
    df = create_dummy_df(freq=freq, periods=periods, ts_type=ts_type, currency="UYU")
    s = Session(location=TEST_DIR, download=False)
    s._datasets["dummy"] = df
    s.convert(flavor="real", start_date=start_date, end_date=end_date)
    s.get("cpi")
    compare = s.datasets["dummy"]
    cpi = s.datasets["cpi"]
    proc_freq = freq

    if freq in ["D", "W", "B", "W-SUN"]:
        if ts_type == "Stock":
            df = df.resample("ME").last()
        else:
            df = df.resample("ME").sum()
        proc_freq = "M"
    save_index = df.index
    cpi = cpi.resample(rule=proc_freq).mean().iloc[:, 0]

    if start_date is None:
        df = df.div(cpi, axis=0)
    elif end_date is None:
        start_date = pd.to_datetime(start_date)
        month = df.index.to_series().sub(start_date).abs().idxmin()
        # month = df.iloc[df.index.get_loc(start_date, method="nearest")].name
        df = df.div(cpi, axis=0) * cpi.loc[month]
    else:
        df = df.div(cpi, axis=0) * cpi[start_date:end_date].mean()
    df = df.reindex(save_index)
    compare.index, compare.columns = df.index, df.columns
    assert df.equals(compare)


@pytest.mark.parametrize(
    "freq,ts_type,periods,cumperiods,currency",
    [
        ("D", "Flujo", 600, 1, "UYU"),
        ("M", "Stock", 48, 1, "USD"),
        ("A-DEC", "Stock", 10, 1, "USD"),
        ("QE-DEC", "Flujo", 24, 4, "UYU"),
    ],
)
def test_convert_gdp(freq, ts_type, periods, cumperiods, currency):
    df = create_dummy_df(
        freq=freq,
        periods=periods,
        ts_type=ts_type,
        currency=currency,
        cumperiods=cumperiods,
    )
    s = Session(location=TEST_DIR, download=False)
    s._datasets["dummy"] = df
    s.convert(flavor="gdp")
    s.get("_monthly_interpolated_gdp")
    compare = s.datasets["dummy"]
    gdp = s.datasets["_monthly_interpolated_gdp"]

    if freq in ["M", "MS"]:
        gdp = gdp.resample(rule=freq).interpolate("linear")
        if cumperiods != 12 and ts_type == "Flujo":
            converter = int(12 / cumperiods)
            df = df.rolling(window=converter).sum()
    elif freq in ["Q", "QE-DEC"]:
        gdp = gdp.resample(freq).asfreq()
        if cumperiods != 4 and ts_type == "Flujo":
            converter = int(4 / cumperiods)
            df = df.rolling(window=converter).sum()
    elif freq in ["A", "A-DEC"]:
        gdp = gdp.resample(freq).asfreq()
    else:
        if ts_type == "Flujo":
            df = df.resample("ME").sum()
        else:
            df = df.resample("ME").mean()
        gdp = gdp.resample(rule="ME").interpolate("linear")

    if currency == "USD":
        gdp = gdp.iloc[:, 1].to_frame()
    else:
        gdp = gdp.iloc[:, 0].to_frame()

    gdp = gdp.reindex(df.index).iloc[:, 0]
    df = df.div(gdp, axis=0).multiply(100)
    compare.index, compare.columns = df.index, df.columns
    div = df / compare
    final = div[((div > 1.01) | (div < 0.99)).any(axis=1)]
    assert len(final) == 0
